import java.awt.*;
import javax.swing.*;
import java.applet.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;

/**
 * Defines an ADT for Attacks.
 * 
 * @see Monster
 */
public class Attack
{
	/**
	 * The Damage associated with the first Attack learned (A1)
	 */
	public static double DAMAGE1 = .1;
	/**
	 * The Damage associated with the second Attack learned (A2)
	 */
	public static double DAMAGE2 = .15;
	/**
	 * The Damage associated with the third Attack learned (A3)
	 */
	public static double DAMAGE3 = .2;
	/**
	 * The Damage associated with the fourth Attack learned (A4)
	 */
	public static double DAMAGE4 = .25;
	/**
	 * The Damage associated with the fifth Attack learned (A5)
	 */
	public static double DAMAGE5 = .4;
	/**
	 * The Damage associated with the sixth Attack learned (A6)
	 */
	public static double DAMAGE6 = .5;
	/**
	 * The Multiplier to use if the Monster is a weak type against the other Monster
	 * 
	 * @see Monster
	 */
	public static final double DAMAGEMULT = 1.15; // If weak against opp type do this times more damage

	/**
	 * The name of the attack
	 */
	private String name;
	/**
	 * The power of the attack, defined above
	 */
	private double power;

	/**
	 * Constructs a new Attack with name and power.
	 * 
	 * @param name The name of the Attack
	 * @param pow the Power of the attack (0<=x<=.5)
	 * @exception InvalidPowerException if pow is not this.DAMAGE*
	 */
	public Attack(String name, double pow) throws InvalidPowerException
	{
		this.name = name;
		if(pow == DAMAGE1 || pow == DAMAGE2 || pow == DAMAGE3 || pow == DAMAGE4 || pow == DAMAGE5 || pow == DAMAGE6)
			power = pow;
		else
			throw new InvalidPowerException(pow + " is not a valid damage amount");
	}

	/**
	 * Get the name associated with the Attack.
	 * 
	 * @return the name of this Attack
	 */
	public String getName() { return name; }
	/**
	 * Gets the power of the Attack
	 * 
	 * @return the power of the Attack
	 */
	public double getPower() { return power; }
	/**
	 * Compares the Object to this.
	 * 
	 * @param o the Object to compare with this
	 * @return true if equal, false otherwise.
	 */
	public boolean equals(Object o)
	{
		if(o instanceof Attack)
		{
			return equals((Attack)o);
		}
		return false;
	}
	/**
	 * Compares the Attack to this. Uses Power to 
	 * determine equality.
	 * 
	 * @param o the Attack to compare with this
	 * @return true if equal, false otherwise.
	 */
	public boolean equals(Attack a)
	{
		return a.power == this.power;
	}
}