import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.awt.image.*;

/**
 * Defines the battling arena.  Displays the Player and opponents health,
 * Handles the use of Items, Switches the Active Monster, and if opponent is
 * a Trainer gives the Player his Badge if Player wins.  The Player recieves
 * half of the money carried by the opponent and recieves experience propertional
 * to the level of the opponent's Monster(s).  Displays the dialogue associated with the 
 * opponent after the conclusion of the battle if Player wins, else Fires a GameOverEvent.
 * 
 * @see Monster
 * @see MapObject
 * @see People
 * @see Person
 * @see Player
 * @see BattleEndEvent
 * @see BattleEndListener
 * @see GameOverEvent
 * @see GameOverListener
 * @see ImageMap
 * @see Item
 * @author Matt, Sam
 * @version 5.4.99.123.456
 */
public class Battle extends Component 
{
	/**
	 * The Color used for the Player's Health bar
	 */
	private static final Color playerHealth = new Color(82, 184, 249);
	/**
	 * The Color used for the opponents Health bar
	 */
	private static final Color opponentHealth = new Color(82, 249, 208);

	/** 
	 * The Player
	 */
	Player p;
	/**
	 * The opponent
	 */
	MapObject opp;

	/**
	 * The ImageMap of the Monsters of the Game
	 */
	ImageMap[] pl; // Add Monster Images
	/**
	 * Location im pl of Player/opponent Active Monster
	 */
	int actp, actop; // Location in arr of Player/Opponent monster

	/**
	 * The BattleEndListeners
	 */
	Vector battleEndListeners;
	/**
	 * The GameOverListeners
	 */
	Vector gameOverListeners;

	/**
	 * If AttackFullException is thrown need
	 * to have the Attack thrown to be abl to add it
	 * if the Player chooses to remove another Attack
	 */
	Attack a;

	/**
	 * Default screen, show options
	 */
	private static final int DEF = 0;
	/**
	 * Player Attack was selected
	 */
	private static final int ATTACKS = 1;
	/**
	 * Player Item was selected
	 */
	private static final int ITEMS = 2;
	/**
	 * Change was selected
	 */
	private static final int CHANGE = 3;
	/**
	 * Dialogue/Information is being shown
	 */
	private static final int DIAG = 4;
	/**
	 * A New Attack is ready to add, but the Monster has too many Attacks
	 */
	private static final int NEWATT = 5;

	private static final String[] def = {"Attacks", "Items", "Change", "Run"};
	/**
	 * Items of the Player
	 */
	private String[] items;
	/**
	 * Monsters of the Player
	 */
	private String[] monsters;
	/**
	 * Attacks of the Active Monster
	 */
	private String[] attacks;
	/**
	 * What is shown now.
	 */
	private String[] shown;

	/** 
	 * Width, Height
	 */
	int w, h;
	/**
	 * Selected, start-offset, displayed
	 */
	int sel, start, disp;
	/**
	 * Player turn over, end the battle, new Attack ready to add
	 */
	boolean pturn, end, natt;

	/**
	 * Construct a Battle between Player and the MapObject.
	 * 
	 * @param p the Player to battle with
	 * @param opp the opponent of the Player
	 * @see Player
	 * @see MapObject
	 * @see Trainer
	 * @see People
	 * @see Person
	 */
	public Battle(Player p, MapObject opp)
	{
		this.p = p;
		this.opp = opp;
		sel = 0;
		start = 0;
		disp = 0;
		shown = new String[def.length];
		for(int i = 0; i < def.length; i++)
			shown[i] = def[i];

		items = p.getItems();
		monsters = p.getMonsters();
		attacks = p.getMonsterAttacks(p.getActiveMonster());

		pturn = false;
		end = false;

		enableEvents( AWTEvent.KEY_EVENT_MASK);
	}
	/**
	 * Construct a Battle between Player and the MapObject using images in the ImageMap.
	 * 
	 * @param p the Player to battle with
	 * @param opp the opponent of the Player
	 * @param monsterImageMap the Images (Front and Back) of all the Monsters used
	 * @see Player
	 * @see MapObject
	 * @see Trainer
	 * @see People
	 * @see Person
	 * @see ImageMap
	 */
	public Battle(Player p, MapObject opp, ImageMap[] monsterImageMap)
	{
		this.p = p;
		this.opp = opp;
		sel = 0;
		start = 0;
		disp = 0;
		shown = new String[def.length];
		for(int i = 0; i < def.length; i++)
			shown[i] = def[i];

		items = p.getItems();
		monsters = p.getMonsters();
		attacks = p.getMonsterAttacks(p.getActiveMonster());

		pl = monsterImageMap;

		actp = find(pl, p.getMonster(p.getActiveMonster()).getName(), true);
		if(opp.getType() == MapObject.PERSON || opp.getType() == MapObject.TRAINER)
			actop = find(pl, ((People)opp).getMonster(((People)opp).getActiveMonster()).getName(), false);
		else if(opp.getType() == MapObject.MONSTER)
			actop = find(pl, ((Monster)opp).getName(), false);
		if(actp == -1) actp = 0;
		if(actop == -1) actop = 0;

		pturn = false;
		end = false;

		enableEvents( AWTEvent.KEY_EVENT_MASK);
	}

	/**
	 * Process the events passed to this.  Overrides the implementation 
	 * of KeyEvent.KEY_PRESSED, and allows super to handle the rest.
	 * 
	 * @param e the AWTEvent to handle
	 */
	public void processEvent(AWTEvent e) 
	{
		int wi = (int)(w*.4)+2;
		int hi = (int)(h*.8);
		int bw = w - wi - 5;
		int bh = (int)(.7*h - 10);

		if(e.getID() == KeyEvent.KEY_PRESSED)
		{
			switch(((KeyEvent)e).getKeyCode())
			{
				case KeyEvent.VK_UP:
				{
					if(disp != ITEMS)
					{
						if(sel > 0)
						{
							if(shown[sel-1] != null)
								sel--;
						}
						else
						{
							if(shown[3] != null)
								sel = 3;
						}
					}
					else
					{
						if(sel == 0)
						{
							if(start == 0)
							{
								if(shown.length - 4 >= 0)
									start = shown.length-4;
								else
									start = 0;
								sel = shown.length-1-start;
							}
							else if( (start - 5) >= 0)
							{
								start -= 5;
								sel = 0;
							}
							else
							{
								start = 0;
								sel = 0;
							}
						}
						else
						{
							if(shown[sel-1] != null)
								sel--;
						}
					}
					break;
				}
				case KeyEvent.VK_DOWN:
					{
					if(disp != ITEMS)
					{
						if(sel < 3)
						{
							if(shown[sel+1] != null)
								sel++;
						}
						else
						{
							if(shown[0] != null)
								sel = 0;
						}
					}
					else
					{
						if((sel+start) == shown.length-1)
						{
							start = 0;
							sel = 0;
						}
						else if(sel == 3)
						{
							if((start+4) < shown.length)
							{
								start += 4;
								sel = 0;
							}
							else
							{
								start = shown.length-5;
								sel = 0;
							}
						}
						else
						{
							if(shown[sel+1] != null)
								sel++;
						}
					}
					break;
				}
				case KeyEvent.VK_ENTER:
				case KeyEvent.VK_SPACE:
					{
					start = 0;
					if(disp == DEF)
					{
						switch(sel)
						{
							case 0:
								{
								sel = 0;
								for(int i = 0; i < attacks.length; i++)
								{
									shown[i] = attacks[i];
								}
								disp = ATTACKS;
								break;
							}
							case 1:
								{
								shown = new String[items.length];
								sel = 0;
								for(int i = start; i < items.length; i++)
								{
									if(items[i] != null)
										shown[i] = items[i] + " x " + p.getItemCount(items[i]);
									else
										shown[i] = null;
								}
								disp = ITEMS;
								break;
							}
							case 2:
								{
								sel = 0;
								for(int i = 0; i < monsters.length; i++)
								{
									shown[i] = monsters[i];
								}
								disp = CHANGE;
								break;
							}
							case 3:
								{
								if(opp == null)
									fireEvent(p);
								else if(opp.getType() != MapObject.TRAINER)
									fireEvent(p); // End Battle
								else
								{
									shown[0] = "Cannot run away from";
									shown[1] = " a trainer battle";
									shown[2] = null;
									shown[3] = null;
									disp = DIAG;
									pturn = false;
								}
								break;
							}
						}
					}
					else if(disp == ATTACKS)
					{
						handleAttack(p, opp, p.getMonster(p.getActiveMonster()).getAttack(sel));
						sel = 0;
						if(!natt)
							disp = DIAG;
						pturn = true;
					}
					else if(disp == CHANGE)
					{
						if(p.getMonster(p.getActiveMonster()).equals(p.getMonster(sel)))
						{
							shown[0] = "Cannot change to th monster ";
							shown[1] = "that is already active";
							shown[2] = null;
							shown[3] = null;
							disp = DIAG;
							pturn = false;
						}
						else
						{
							p.setActiveMonster(sel);
							attacks = p.getMonsterAttacks(p.getActiveMonster());

							Monster m = p.getMonster(p.getActiveMonster());

							sel = 0;
							shown[0] = p.getName() + " changed active monster to:";
							shown[1] = m.getName();
							shown[2] = null;
							disp = DIAG;
							pturn = true;
							actp = find(pl, p.getMonster(p.getActiveMonster()).getName(), true);
							repaint();
						}
					}
					else if(disp == ITEMS)
					{
						if(shown[sel+start] == null)
							return;
						if(handleItem(p.getItem(shown[sel+start].substring(0, shown[sel+start].indexOf(" x")))))
						{
							pturn = true;
						}
						else
						{
							pturn = false;
							// Redo turn
						}
						sel = 0;
						start = 0;
						disp = DIAG;
					}
					else if(disp == DIAG)
					{
						if(!pturn)
						{
							sel = 0;
							for(int i = 0; i < def.length; i++)
							{
								shown[i] = def[i];
							}
							disp = DEF;
							pturn = false;
						}
						else if(end)
						{
							disp = DEF;
							pturn = false;
							repaint();
							fireEvent(p);
						}
						else
						{
							if(opp.getType() == MapObject.MONSTER)
							{
								Attack a = BattleAI.recommendMonsterAttack((Monster)opp, p.getMonster(p.getActiveMonster()));
								handleAttack(opp, p, a);
							}
							else
							{
								try
								{
									Attack a = BattleAI.recommendPlayerAttack((People)opp, p);
									handleAttack(opp, p, a);
								}
								catch(ItemChangeException ie)
								{
									Item it = ie.getItem();
									Monster mon = ((People)opp).getMonster(((People)opp).getActiveMonster());
									((People)opp).useItem(it, mon);

									shown[0] = ((People)opp).getName() + " used " + it.getName() + " on ";
									shown[1] = mon.getName();
									shown[2] = null;
									shown[3] = null;
								}
								catch(MonsterChangeException me)
								{

									((People)opp).setActiveMonster(me.getNewMonster());

									Monster m = ((People)opp).getMonster(((People)opp).getActiveMonster());

									shown[0] = ((People)opp).getName() + " changed active monster to:";
									shown[1] = m.getName();
									shown[2] = null;
									disp = DIAG;
									pturn = true;
									actop = find(pl, ((People)opp).getMonster(((People)opp).getActiveMonster()).getName(), false);
									repaint();
								}
							}
							disp = DIAG;
							pturn = false;
						}
					}
					else if(disp == NEWATT)
					{
						if(natt)
						{
							sel = 0;
							for(int i = 0; i < attacks.length; i++)
							{
								shown[i] = attacks[i];
							}
							disp = NEWATT;
							natt = false;
						}
						else
						{
							Monster m = p.getMonster(p.getActiveMonster());
							Attack ab = m.getAttack(sel);
							m.substituteAttack(ab, a);
							shown[0] = m.getName() + " Learned Attack:";
							shown[1] = a.getName() + "!";
							shown[2] = null;
							shown[3] = null;
							attacks = p.getMonsterAttacks(p.getActiveMonster());
							disp = DIAG;
							pturn = true;
							sel = 0;
						}
					}
					break;
				}
				case KeyEvent.VK_ESCAPE:
				{
					start = 0;
					if(disp != DIAG)
					{
						sel = 0;
						for(int i = 0; i < def.length; i++)
						{
							shown[i] = def[i];
						}
						disp = DEF;
						pturn = false;
					}
					break;
				}
			}
			repaint(bw, bh, wi, hi);
		}
		else if(((KeyEvent)e).getID() == KeyEvent.KEY_RELEASED) { }
		super.processEvent(e);
	}

	/**
	 * Draws the Battle screen system with Monsters, Health bars, and Menus
	 * 
	 * @param g the Graphics
	 */
	public synchronized void paint(Graphics g)
	{
		w = getWidth();
		h = getHeight();

		g.setColor(Color.white);
		g.fillRect(0, 0, w, h);

		if(pl != null)
			g.drawImage(pl[actp].getImage(), 5, h - pl[actp].getImage().getHeight(this) + 5, this);
		if(pl[actop] != null)
			g.drawImage(pl[actop].getImage(), w - pl[actop].getImage().getWidth(this) - 5, 5, this);
		drawMenu(g);
		if(opp != null && p != null && g != null)
			drawHealth(p, opp, g);
	}
	/**
	 * Draws the Menu of options or the dialogue.
	 * 
	 * @param g the Graphics
	 */
	private synchronized void drawMenu(Graphics g)
	{
		int wi = (int)(w*.4);
		int hi = (int)(h*.2);
		int bw = w - wi - 5;
		int bh = h - hi - 5;

		g.setColor(Color.white);
		g.fillRect(bw, bh, wi, hi);

		g.setColor(Color.black);
		g.drawRect(bw, bh, wi, hi);

		if(shown == null)
			return;

		int posh = 0;
		int he = hi/4;
		for(int i = start, j = 0; i < shown.length && shown[i] != null && j < 4; i++, j++)
		{
			if(j == sel && disp != DIAG && !natt)
			{
				g.setColor(Color.red);
			}
			else g.setColor(Color.black);
			g.drawString(shown[i], bw+7, bh+posh+15);
			posh += hi/4;
		}

		if(disp != DIAG && !natt)
		{
			g.setColor(Color.red);
			g.drawLine(bw+7, bh + sel*he+15+3, bw+wi-7, bh + sel*he+15+3);
		}
	}
	/**
	 * Draws the Health bars of the Player and opponent.
	 * 
	 * @param pl the Player
	 * @param op the opponent
	 * @param g the Graphics
	 */
	private synchronized void drawHealth(Player pl, MapObject op, Graphics g)
	{
		int wi = (int)(w*.4);
		int hi = (int)(h*.05);
		int bw = w - wi - 5;
		int bh = h - (int)(h*.2) - hi - 10;
		int eh = h - (int)(h*.2) - hi*2 - 10;

		Monster m = pl.getMonster(pl.getActiveMonster()); // Player
		if(m != null)
		{
			int hp = m.getHP();
			int mhp = m.getMaxHP();
			double hpend = (double)wi*(double)hp/(double)mhp;

			g.setColor(playerHealth);
			g.fillRect(bw, bh, (int)hpend, hi);
			g.setColor(Color.black);
			g.drawRect(bw, bh, wi, hi);
			g.drawString(pl.getName() + ": " + m.getName() + ", L: " + m.getLevel(), bw + 5, bh + (int)(hi/2));
		}

		String output = "";
		if(op.getType() == MapObject.PERSON || op.getType() == MapObject.TRAINER)
		{
			m = ((People)op).getMonster(((People)op).getActiveMonster()); // Opponent
			if(m != null)
				output = opp.getName() + ": " + m.getName() + ", L: " + m.getLevel();
		}
		else if(op.getType() == MapObject.MONSTER)
		{
			m = (Monster)op;
			output = m.getName() + ", L: " + m.getLevel();
		}
		if(m != null)
		{
			int hp = m.getHP();
			int mhp = m.getMaxHP();
			double hpend = (double)wi*(double)hp/(double)mhp;

			g.setColor(opponentHealth);
			g.fillRect(bw, eh, (int)hpend, hi);
			g.setColor(Color.black);
			g.drawRect(bw, eh, wi, hi);
			g.drawString(output, bw + 5, eh + (int)(hi/2));
		}
	}
	/**
	 * Adds a BattleEndListener to be informed.
	 * 
	 * @param bl the Class to be informed of BattleEndEvents.
	 * @see BattleEndEvent
	 * @see BattleEndListener
	 */
	public void addBattleEndListener(BattleEndListener bl)
	{
		if ( battleEndListeners == null )
			battleEndListeners = new Vector();
		battleEndListeners.addElement( bl );
	}
	/**
	 * Removes a BattleEndListener to not be informed anymore.
	 * 
	 * @param bl the Class to stop being informed of BattleEndEvents.
	 * @see BattleEndEvent
	 * @see BattleEndListener
	 */
	public void removeBattleListener(BattleEndListener bl)
	{
		if ( battleEndListeners != null )
			battleEndListeners.removeElement( bl );
	}
	/**
	 * Fires the BattleEndEvent to all listeners
	 * 
	 * @param mp the MapObject to pass along.
	 * @see BattleEndEvent
	 * @see BattleEndListener
	 */
	private void fireEvent(MapObject mp) 
	{
		if ( battleEndListeners == null )
			return;
		BattleEndEvent event = new BattleEndEvent(this, BattleEndEvent.BATTLE_ENDED, p);
		for (Enumeration e = battleEndListeners.elements(); e.hasMoreElements(); )
			((BattleEndListener)e.nextElement()).battleEnd( event );
	}
	/**
	 * Adds a GameOverListener to be informed.
	 * 
	 * @param bl the Class to be informed of GameOverEvents.
	 * @see GameOverEvent
	 * @see GameOverListener
	 */
	public void addGameOverListener(GameOverListener bl)
	{
		if ( gameOverListeners == null )
			gameOverListeners = new Vector();
		gameOverListeners.addElement( bl );
	}
	/**
	 * Removes a GameOverListener to stop being informed.
	 * 
	 * @param bl the Class to stop being informed of GameOverEvents.
	 * @see GameOverEvent
	 * @see GameOverListener
	 */
	public void removeGameOverListener(GameOverListener bl)
	{
		if ( gameOverListeners != null )
			gameOverListeners.removeElement( bl );
	}
	/**
	 * Fires a GameOverEvent to all Listeners
	 * 
	 * @see GameOverEvent
	 * @see GameOverListener
	 */
	private void fireGameOverEvent() 
	{
		if ( gameOverListeners == null )
			return;
		GameOverEvent event = new GameOverEvent(this, GameOverEvent.GAME_OVER);
		for (Enumeration e = gameOverListeners.elements(); e.hasMoreElements(); )
			((GameOverListener)e.nextElement()).gameOver( event );
	}
	/**
	 * Handle the attack choice between the attacker, reciever using Attack pla.
	 * 
	 * @param attacker the MapObject attacking
	 * @param reciever the MapObject being attacked
	 * @param pla the Attack being used
	 * @see MapObject
	 * @see Attack
	 * @see Trainer
	 * @see Player
	 * @see Person
	 * @see People
	 * @see Monster
	 */
	private void handleAttack(MapObject attacker, MapObject reciever, Attack pla)
	{
		Player p = null;
		People op = null;
		Monster pm = null, om = null;
		String attName = null;
		if(attacker.getType() == MapObject.PLAYER)
		{
			p = (Player)attacker;
			pm = p.getMonster(p.getActiveMonster());
			attName = p.getName();
		}
		else if(attacker.getType() == MapObject.PERSON || attacker.getType() == MapObject.TRAINER)
		{
			op = (People)attacker;
			pm = op.getMonster(op.getActiveMonster());
			attName = op.getName();
		}
		else if(attacker.getType() == MapObject.MONSTER)
		{
			pm = (Monster)attacker;
			attName = pm.getName();
		}
		if(reciever.getType() == MapObject.PERSON || reciever.getType() == MapObject.TRAINER)
		{
			op = (People)reciever;
			om = op.getMonster(op.getActiveMonster());
		}
		else if(reciever.getType() == MapObject.PLAYER)
		{
			p = (Player)reciever;
			om = p.getMonster(p.getActiveMonster());
		}
		else if(reciever.getType() == MapObject.MONSTER)
		{
			om = (Monster)reciever;
		}

		if(pm != null && om != null)
		{
			double pa = pla.getPower();
			int damage;
			int rhp = om.getHP();
			int min = Math.abs((pm.getLevel() - om.getLevel()) * (pm.getLevel() / 25) / 3);
			min = (min == 0 ? 2 : min);
			if(isWeakAgainst(pm.getKind(), om.getKind()))
			{
				damage = (int)( (2*pm.getAttack()*Attack.DAMAGEMULT - .5*om.getDefense())*pa *(pm.getLevel()/om.getLevel()));
				if(damage > min)
				{
					om.setHP(rhp - damage);
				}
				else
				{
					damage = (int)(rhp/64);
					om.setHP(rhp - (damage > min ? damage : min));
				}
			}
			else
			{
				damage = (int)( (2*pm.getAttack() - .5*om.getDefense())*pa *(pm.getLevel()/om.getLevel()));
				if(damage > min)
				{
					om.setHP(rhp - damage);
				}
				else
				{
					damage = (int)(rhp/64);
					damage = (damage > 0 ? damage : 2);
					om.setHP(rhp - damage);
				}
			}

			if(attacker.getType() == MapObject.MONSTER)
				shown[0] = pm.getName() + " did:";
			else
				shown[0] = attName + "'s " + pm.getName() + " did:";
			shown[1] = damage + " damage to " + om.getName() + (om.getHP() == 0 ? " killing him." : "" );
			shown[2] = null;
			shown[3] = null;
			if(pa == pla.DAMAGE1)
			{
				om.setDefense(om.getDefense()-5);
				shown[2] = "and lowered " + om.getName() + " Defense by 5";
			}
			else if(pa == pla.DAMAGE3)
			{
				om.setAttack(om.getAttack() -8);
				shown[2] = "and lowered " + om.getName() + " Attack by 8";
			}
			else if(pa == pla.DAMAGE4)
			{
				om.setDefense(om.getDefense()-5);
				om.setAttack(om.getAttack() -8);
				pm.setHP(pm.getHP() - (int)(pm.getHP()*.01));
				shown[2] = "and lowered " + om.getName() + " Defense by 5, Attack by 8";
				shown[3] = "and lowered " + pm.getName() + " to: " + pm.getHP();
			}
			else if(pa == pla.DAMAGE5)
			{
				om.setDefense(om.getDefense()-8);
				om.setAttack(om.getAttack() -9);
				pm.setHP(pm.getHP() - (int)(pm.getHP()*.05));
				shown[2] = "and lowered " + om.getName() + " Defense by 8, Attack by 9";
				shown[3] = "and lowered " + pm.getName() + " to: " + pm.getHP();
			}

			if(om.getHP() == 0)
			{
				if(reciever.getType() == MapObject.PERSON || reciever.getType() == MapObject.TRAINER)
				{
					if(op.getNumberOfMonsters() == 0)
					{
						p.stopBattle();
						givePlayerExperience(pm, om);
						finishFight(om);
						if(!natt)
						{
							shown[0] = ((People)reciever).getDialogue(); // Add real Dialogue
							shown[1] = "You defeated my monsters,";
							shown[2] = "I will have my revenge!!";;
							shown[3] = null;
						}
						end = true;
					}
					else
					{
						givePlayerExperience(pm, om);
						op.setActiveMonster(0);
						actop = find(pl, ((People)opp).getMonster(((People)opp).getActiveMonster()).getName(), false);
						repaint();
						// Force Opp change
					}
				}
				else if(reciever.getType() == MapObject.MONSTER)
				{
					p.stopBattle();
					givePlayerExperience(pm, om);
					finishFight(om);
					if(!natt)
					{
						shown[0] = "Arrgh!"; // Add real Dialogue
						shown[1] = reciever.getName();
						shown[2] = null;
						shown[3] = null;
					}
					end = true;
				}
				else
				{
					if(p.getNumberOfMonsters() == 0)
					{
						fireGameOverEvent();
					}
					else
					{
						p.setActiveMonster(0);

						monsters = p.getMonsters();
						attacks = p.getMonsterAttacks(0);
						actp = find(pl, p.getMonster(p.getActiveMonster()).getName(), true);
						repaint();
					}
				}
			}
		}
	}
	/**
	 * If the Player's active Monster won give him the experience propertional to the difference between
	 * his level and the opponent's Monster level.
	 * 
	 * @param pm the Players Monster
	 * @param om the opponents Monster
	 * @see Monster
	 */
	private void givePlayerExperience(Monster pm, Monster om)
	{
		try
		{
			if(pm.finishFight(om))
			{
				attacks = p.getMonsterAttacks(p.getActiveMonster());
				shown[0] = pm.getName() + " leveled up to level" + pm.getLevel();
				shown[1] = "Now has attacks:" ;
				shown[2] = "";
				shown[3] = "";
				for(int i = 0; i < 2 && attacks[i] != null; i++)
				{
					shown[2] += attacks[i] + " , ";
				}
				for(int i = 2; i < attacks.length && attacks[i] != null; i++)
				{
					shown[3] += attacks[i] + " , ";
				}
			}
			pturn = true;
		}
		catch(AttacksFullException e)
		{
			a = e.getAttack();
			natt = true;
			disp = NEWATT;
			shown[0] = pm.getName() + " is trying to learn ";
			shown[1] = a.getName() + " Press enter or space to";
			shown[2] = "select an attack to forget,";
			shown[3] = "or press escape to keep attacks known";
		}
	}
	/**
	 * Do the cleanup to finish a fight.
	 * 
	 * @paramom th opponents Monster
	 */
	private void finishFight(Monster om)
	{
		if(opp.getType() == MapObject.PERSON || opp.getType() == MapObject.TRAINER)
		{
			People op = (People)opp;
			if(op.getType() == MapObject.TRAINER)
			{
				String b = ((Trainer)op).getBadge();
				p.addBadge(b);
				((Trainer)op).setBadge("");
				shown[3] = "You earned Badge: " + b;
			}
			int mon = ((int)(op.getMoney())/2);
			p.setMoney(p.getMoney() + mon);
			op.setMoney(op.getMoney() - mon);
			op.beat();
		}
	}
	/**
	 * Determines if a Monster is weak against the other.
	 * 
	 * @param plt the Monster to check against
	 * @param opt the Monster to check
	 * @see Attack
	 * @see Monster
	 */
	private boolean isWeakAgainst(int plt, int opt) // If opt < plt
	{
		switch(plt)
		{
			case Monster.GAS:
			{
				if(opt == Monster.ELECTRIC || opt == Monster.EARTH)
					return true;
				return false;
			}
			case Monster.FIRE:
			{
				if(opt == Monster.GAS || opt == Monster.ELECTRIC)
					return true;
				return false;
			}
			case Monster.WATER:
			{
				if(opt == Monster.GAS || opt == Monster.FIRE)
					return true;
				return false;
			}
			case Monster.EARTH:
			{
				if(opt == Monster.WATER || opt == Monster.FIRE)
					return true;
				return false;
			}
			case Monster.ELECTRIC:
			{
				if(opt == Monster.EARTH || opt == Monster.WATER)
					return true;
				return false;
			}
		}
		return false;
	}
	/**
	 * Use the Item that was selected, if possible.  For example
	 * MonsterBalls cannot be used on already captured Monsters.
	 * 
	 * @param it the Item to use
	 * @see Item
	 * @see Monster
	 */
	private boolean handleItem(Item it)
	{
		if(it == null)
			return false;
		shown = new String[4];
		if(it.getBrand() == Item.MONSTERBALL)
		{
			if(opp.getType() == MapObject.PERSON || opp.getType() == MapObject.TRAINER)
			{
				shown[0] = "Cannot use Monster Balls on monsters";
				shown[1] = "have been captured by people";
				shown[2] = null;
				shown[3] = null;
				return false;
			}
			else
			{
				Monster mon = (Monster)opp;
				if(it.useItem(mon))
				{
					if(p.addMonster(mon))
					{
						shown[0] = mon.getName() + " was captured and added to";
						shown[1] = p.getName() + "'s hand using the";
						shown[2] = it.getName();
						shown[3] = null;
					}
					else
					{
						shown[0] = mon.getName() + " was captured and added to";
						shown[1] = p.getName() + "'s home collection using the";
						shown[2] = it.getName();
						shown[3] = null;
					}
					opp = null;
					end = true;
				}
				else
				{
					shown[0] = mon.getName() + " evaded capture narrowly avoiding";
					shown[1] = "the " + it.getName();
					shown[2] = null;
					shown[3] = null;
				}
			}
		}
		else if(it.getBrand() == Item.POTION)
		{
			Monster mon = p.getMonster(p.getActiveMonster());
			p.useItem(it, mon);
			items = p.getItems();
			shown[0] = p.getName() + " used " + it.getName() + " on ";
			shown[1] = mon.getName();
			shown[2] = null;
			shown[3] = null;
		}
		return true;
	}
	/**
	 * Find the index in the ImageMap array that the MapObject's
	 * Image is at.
	 * 
	 * @param im th ImageMap  array
	 * @param n the Name of the Monster
	 * @param player true iff Player false otherwise
	 */
	private int find(ImageMap[] im, String n, boolean player)
	{
		for(int i = 0; i < im.length; i++)
		{
			if(im[i].equals(n, player))
				return i;
		}
		System.out.println("|" + n + "|");
		return -1;
	}
}