import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.util.*;

/**
 * The BattleEvent for when Battle's start
 * 
 * @see Grid
 * @see Battle
 * @see MapObject
 * @see Monster
 * @see Player
 * @see Person
 * @see Trainer
 * @see People
 */
public class BattleEvent extends AWTEvent 
{
	/**
	 * The opponnent
	 */
	MapObject opp;
	/**
	 * The Player
	 */
	Player pl;
	/**
	 * The UID of this Event
	 */
	public static final int BATTLE_STARTED = RESERVED_ID_MAX + 1;	// Make sure no conflicts with normal AWT components

	/**
	 * Constructs a new BattleEvent.
	 * 
	 * @param source the Grid this event was fired from
	 * @param id the ID
	 * @param mp1 the MapObject #1 to be passed along
	 * @param mp2 the MapObject #2 to be passed along
	 */
	BattleEvent(Grid source, int id, MapObject mp1, MapObject mp2)
	{
		super( source, id );
		if(mp1.getType() == MapObject.PLAYER)
		{
			pl = (Player)mp1;
			opp = mp2;
		}
		else
		{
			pl = (Player)mp2;
			opp = mp1;
		}
	}
	/**
	 * Returns the Player of this
	 * 
	 * @return the Player
	 */
	public Player getPlayer()
	{
		return pl;
	}
	/**
	 * Returns the Opponent of the Player
	 * 
	 * @return the MapObject of the Opponent
	 */
	public MapObject getOpponent()
	{
		return opp;
	}
}
