import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Simple class for displaying an image
 * 
 * @see Game
 * @version 1
 * @author Matt, Sam
 */
class Splash extends Component
{
	/**
	 * The Image to display
	 */
	Image img;
	/**
	 * Construct a new Splash with the image.
	 * 
	 * @param img the Image to paint
	 */
	Splash(Image img)
	{
		this.img = img;
	}
	/**
	 * Draw the image
	 * 
	 * @param g the Graphics
	 */
	public void paint(Graphics g)
	{
		g.drawImage(img, 0, 0, this);
	}
}

/*
 * <applet code="Game" width = 640 height = 450>
 * </applet>
 */
/**
 * Main class to handle the switching between game modes, and initiation of the game.
 * 
 * @see Grid
 * @see Battle
 * @see Shop
 * @see PlayerLoader
 * @see PlayerSaver
 * @see MapLoader
 * @version 500
 * @author Matt, Sam
 */
public class Game extends Applet implements ActionListener, BattleListener, BattleEndListener, RedirectListener, ShopEndListener, GameOverListener, SaveListener
{
	Grid grid;
	Battle battle;
	Shop shop;
	MapLoader mp;
	Splash sp;
	Panel spp;
	Button bn, bl, bname, ok;
	TextField tname;
	TextArea savedata;
	boolean num;
	int win;

	Vector mc;
	// Need one for each type to instanciate the images
	// STUPID Applet Security Model!!
	// Should be able to read from directory that exists in!!
	Person p;
	Trainer t;
	Player pl;
	Grass g;
	Rock r;
	Floor f;
	Building b;
	Wall w;
	Redirect re;
	Image /*pf, pef, tef,*/ sh, splash, gover, gwin;
	ImageMap[] monsters;
	Graphics graph;

	/**
	 * Starts the game displaying the default Splash Screen to the user.
	 */
	public void init()
	{
		num = true;
		win = -1;
		setLayout(new BorderLayout());

		setUpImages();

		shop = new Shop(pl, sh);
		shop.addShopEndListener(this);
		shop.addSaveListener(this);

		sp = new Splash(splash);
		spp = new Panel();
		bn = new Button("New Game");
		bn.addActionListener(this);
		bl = new Button("Load Game");
		bl.addActionListener(this);
		ok = new Button("OK");
		ok.addActionListener(this);
		spp.add(bn);
		spp.add(bl);
		add(sp, BorderLayout.CENTER);
		add(spp, BorderLayout.SOUTH);

		bname = new Button("Play");
		bname.addActionListener(this);
		tname = new TextField(15);
		tname.addActionListener(this);

		savedata = new TextArea();

	}

	/**
	 * Handle a battleBegin Event, ie pause Grid,
	 * display the Battle system and move the MapObjects
	 * between the two.
	 * 
	 * @see MapObject
	 * @see Grid
	 * @see Battle
	 * @see Player
	 * @see People
	 * @see Monster
	 * 
	 * @param e the BattleEvent
	 */
	public void battleBegin(BattleEvent e)
	{
		MapObject m = e.getOpponent();
		grid.transferFocus();

		// IF'd b/c using varying images
		if(m.getType() == MapObject.PERSON)
		{
			battle = new Battle(e.getPlayer(), e.getOpponent(), monsters);
		}
		else if(m.getType() == MapObject.TRAINER)
		{
			battle = new Battle(e.getPlayer(), e.getOpponent(), monsters);
		}
		else if(m.getType() == MapObject.MONSTER)
		{
			battle = new Battle(e.getPlayer(), e.getOpponent(), monsters);
		}
		battle.addBattleEndListener(this);
		battle.addGameOverListener(this);
		removeAll();
		add(battle, BorderLayout.CENTER);
		battle.requestFocus();
		battle.setSize(getWidth(), getHeight());
		battle.paintAll(graph);
	}
	/**
	 * Handle a battleEnd Event, ie unpause Grid,
	 * display the Grid system, destroy Battle system and move the MapObjects
	 * between the two.
	 * 
	 * @see MapObject
	 * @see Grid
	 * @see Battle
	 * @see Player
	 * @see People
	 * @see Monster
	 * 
	 * @param e the BattleEndEvent
	 */
	public void battleEnd(BattleEndEvent e)
	{
		pl = e.getPlayer();
		pl.stopBattle();
		battle.transferFocus();
		grid.setPlayer(pl);
		grid.fillMonsters();
		removeAll();
		add(grid, BorderLayout.CENTER);
		grid.requestFocus();
		grid.setSize(getWidth(), getHeight());
		grid.paintAll(graph);
	}
	/**
	 * Handle a shopEnd Event, ie display the Shop, and move 
	 * Player between the two
	 * 
	 * @see MapObject
	 * @see Grid
	 * @see Battle
	 * @see Player
	 * @see People
	 * @see Monster
	 * 
	 * @param e the ShopEndEvent
	 */
	public void shopEnd(ShopEndEvent e)
	{
		shop.transferFocus();
		pl = e.getPlayer();
		pl.stopBattle();
		grid.fillMonsters();
		grid.setPlayer(pl);
		removeAll();
		add(grid, BorderLayout.CENTER);
		grid.requestFocus();
		grid.setSize(getWidth(), getHeight());
		grid.paintAll(graph);
	}
	/**
	 * Handle a redirect event, ie load the new map
	 * specified in the RedirectEvent.  If the map specified
	 * is Shop load the Shop system.
	 * 
	 * @see MapObject
	 * @see Grid
	 * @see RedirectEvent
	 * @see MapLoader
	 * 
	 * @param e the RedirectEvent
	 */
	public void redirectFired(RedirectEvent e)
	{
		String name = e.getMapName();
		pl = e.getPlayer();
		pl.stopBattle();
		if(name.equalsIgnoreCase("ItemShop"))
		{
			shop = new Shop(pl, sh);
			shop.addShopEndListener(this);
			shop.addSaveListener(this);
			pl.battle();
			removeAll();
			add(shop, BorderLayout.CENTER);
			shop.requestFocus();
			shop.setSize(getWidth(), getHeight());
			shop.paintAll(graph);
			return;
		}
		else if(name.equalsIgnoreCase("Winner"))
		{
			sp = new Splash(gwin);
			removeAll();
			add(sp, BorderLayout.CENTER);
			add(ok, BorderLayout.SOUTH);
			win = 0;
			paintAll(graph);
			return;
		}
		try
		{
			URL url = new URL(getCodeBase(), "maps/" + name + ".mml");
			URLConnection uc = url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
			mp = new MapLoader(in);
		}
		catch(Exception ee) { ee.printStackTrace(); }
		synchronized(grid)
		{
			grid.destroy();
			removeAll();
			grid = mp.getGrid();
			grid.addBattleListener(this);
			grid.addRedirectListener(this);
			grid.init();
			grid.setPlayer(pl);
			add(grid, BorderLayout.CENTER);
			grid.requestFocus();
			grid.setSize(getWidth(), getHeight());
			grid.paintAll(graph);
		}
	}
	/**
	 * Handle a GameOver Event, ie display the Game Over splash
	 * and offer the player the option to rebegin.
	 * 
	 * @see MapObject
	 * @see Grid
	 * @see Battle
	 * @see Player
	 * @see People
	 * @see Monster
	 * 
	 * @param e the GameOverEvent
	 */
	public void gameOver(GameOverEvent e)
	{
		removeAll();
		sp = new Splash(gover);
		spp.removeAll();
		spp.add(bn);
		spp.add(bl);
		add(sp, BorderLayout.CENTER);
		add(spp, BorderLayout.SOUTH);
		sp.setSize(getWidth(), getHeight());
		paintAll(graph);
	}
	/**
	 * Handle a SaveGame Event, ie take the Player
	 * specified in SaveEvent and pass it to PlayerSaver
	 * then place this text in a TextArea for the user to save it.
	 * 
	 * @see MapObject
	 * @see PlayerSaver
	 * @see SaveEvent
	 * @see Player
	 * @see People
	 * @see Monster
	 * 
	 * @param e the SaveEvent
	 */
	public void saveGame(SaveEvent e)
	{
		savedata.setEditable(false);
		removeAll();
		PlayerSaver ps = new PlayerSaver(e.getPlayer());
		savedata.setText(ps.toString());
		add(savedata, BorderLayout.CENTER);
		add(ok, BorderLayout.SOUTH);
		add(new Label("Copy the text below and save it in a text file for loading later", Label.CENTER), BorderLayout.NORTH);
		paintAll(graph);
	}
	/**
	 * Handle Button clicks to load, start a new game, etc.
	 * 
	 * @param e the ActionEvent
	 */
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == bn)
		{
			removeAll();
			spp = new Panel();
			spp.add(new Label("Please Enter Your Name"));
			spp.add(tname);
			spp.add(bname);
			add(sp, BorderLayout.CENTER);
			add(spp, BorderLayout.SOUTH);
			tname.requestFocus();
			paintAll(graph);
		}
		else if(e.getSource() == bname || e.getSource() == tname)
		{
			String name = tname.getText();
			if(name.equals(""))
				name = "Player1";
			// Setup pl for def temp

			loadPlayerFile("Default");

			pl.setName(name);
			removeAll();
			try
			{
				URL url = new URL(getCodeBase(), "maps/Saphire.mml");
				URLConnection uc = url.openConnection();
				BufferedReader in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
				mp = new MapLoader(in);
			}
			catch(Exception ee) { ee.printStackTrace(); }
			initGrid();
		}
		else if(e.getSource() == ok)
		{
			removeAll();
			savedata.setText("");
			if(win == 0)
			{
				savedata.setEditable(false);
				removeAll();
				PlayerSaver ps = new PlayerSaver(pl);
				savedata.setText(ps.toString());
				add(savedata, BorderLayout.CENTER);
				add(ok, BorderLayout.SOUTH);
				add(new Label("Copy the text below and save it in a text file for loading later", Label.CENTER), BorderLayout.NORTH);
				paintAll(graph);
				win = 1;
			}
			else if(win == 1)
			{
				removeAll();
				spp.removeAll();
				sp = new Splash(splash);
				spp.add(bn);
				spp.add(bl);
				add(sp, BorderLayout.CENTER);
				add(spp, BorderLayout.SOUTH);
				win = -1;
				paintAll(graph);
			}
			else
			{
				add(shop, BorderLayout.CENTER);
				shop.requestFocus();
				shop.setSize(getWidth(), getHeight());
				shop.repaint();
			}
		}
		else if(e.getSource() == bl)
		{
			if(num)
			{
				savedata.setEditable(true);
				removeAll();
				savedata.setText("");
				add(savedata, BorderLayout.CENTER);
				add(bl, BorderLayout.SOUTH);
				add(new Label("Paste the text you copied and saved earlier below", Label.CENTER), BorderLayout.NORTH);
				paintAll(graph);
				num = false;
				savedata.requestFocus();
			}
			else
			{
				try
				{
					BufferedReader in = new BufferedReader(new StringReader(savedata.getText()));
					PlayerLoader pload = new PlayerLoader(in);
					pl = pload.getPlayer();
				}
				catch(Exception ee) 
				{
					removeAll();
					spp.removeAll();
					spp.add(bn);
					spp.add(bl);
					add(sp, BorderLayout.CENTER);
					add(spp, BorderLayout.SOUTH);
					add(new Label("Invalid Player Save File", Label.CENTER), BorderLayout.NORTH);
					paintAll(graph);
					return;
				}

				num = true;
				removeAll();
				try
				{
					URL url = new URL(getCodeBase(), "maps/Saphire.mml");
					URLConnection uc = url.openConnection();
					BufferedReader in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
					mp = new MapLoader(in);
				}
				catch(Exception ee) { ee.printStackTrace(); }
				initGrid();
			}
		}
	}
	/**
	 * Draw the Components
	 */
	public void paint(Graphics g)
	{
		g.drawString("Loading...", getWidth()/2, getHeight()/2);
		this.graph = g;
		super.paint(g);
	}
	/**
	 * Load all the required Images and instantiate the
	 * classes that depend on them.
	 */
	public void setUpImages()
	{
		Image[] img = new Image[4];
		img[0] = getImage(getCodeBase(), "images/PersonF-N.gif");
		img[1] = getImage(getCodeBase(), "images/PersonB-N.gif");
		img[2] = getImage(getCodeBase(), "images/PersonL-N.gif");
		img[3] = getImage(getCodeBase(), "images/PersonR-N.gif");
		p = new Person();
		p.setImage(img);

		img[0] = getImage(getCodeBase(), "images/TrainerF.gif");
		img[1] = getImage(getCodeBase(), "images/TrainerB.gif");
		img[2] = getImage(getCodeBase(), "images/TrainerL.gif");
		img[3] = getImage(getCodeBase(), "images/TrainerR.gif");
		t = new Trainer();
		t.setImage(img);

		img[0] = getImage(getCodeBase(), "images/PersonF-Still.gif");
		img[1] = getImage(getCodeBase(), "images/PersonB-Still.gif");
		img[2] = getImage(getCodeBase(), "images/PersonL-Still.gif");
		img[3] = getImage(getCodeBase(), "images/PersonR-Still.gif");
		pl = new Player();
		pl.setImage(img);

		img = new Image[1];
		img[0] = getImage(getCodeBase(), "images/Grass.gif");
		g = new Grass();
		g.setImage(img);

		img[0] = getImage(getCodeBase(), "images/Rock.gif");
		r = new Rock();
		r.setImage(img);

		img[0] = getImage(getCodeBase(), "images/Floor.gif");
		f = new Floor();
		f.setImage(img);

		img[0] = getImage(getCodeBase(), "images/Building.gif");
		b = new Building();
		b.setImage(img);

		img[0] = getImage(getCodeBase(), "images/Wall.gif");
		w = new Wall();
		w.setImage(img);

		img[0] = getImage(getCodeBase(), "images/Redirect.gif");
		re = new Redirect();
		re.setImage(img);

		monsters = new ImageMap[20];
		monsters[0] = new ImageMap("Alligatrix-B", getImage(getCodeBase(), "images/Alligatrix-B.jpg"));
		monsters[1] = new ImageMap("Alligatrix-F", getImage(getCodeBase(), "images/Alligatrix-F.jpg"));

		monsters[2] = new ImageMap("Eurekasaur-B", getImage(getCodeBase(), "images/Eurekasaur-B.jpg"));
		monsters[3] = new ImageMap("Eurekasaur-F", getImage(getCodeBase(), "images/Eurekasaur-F.jpg"));

		monsters[4] = new ImageMap("Flamesaur-B", getImage(getCodeBase(), "images/Flamesaur-B.jpg"));
		monsters[5] = new ImageMap("Flamesaur-F", getImage(getCodeBase(), "images/Flamesaur-F.jpg"));

		monsters[6] = new ImageMap("Groundyx-B", getImage(getCodeBase(), "images/Groundyx-B.jpg"));
		monsters[7] = new ImageMap("Groundyx-F", getImage(getCodeBase(), "images/Groundyx-F.jpg"));

		monsters[8] = new ImageMap("MoPyro-B", getImage(getCodeBase(), "images/MoPyro-B.jpg"));
		monsters[9] = new ImageMap("MoPyro-F", getImage(getCodeBase(), "images/MoPyro-F.jpg"));

		monsters[10] = new ImageMap("Rhono-B", getImage(getCodeBase(), "images/Rhono-B.jpg"));
		monsters[11] = new ImageMap("Rhono-F", getImage(getCodeBase(), "images/Rhono-F.jpg"));

		monsters[12] = new ImageMap("Sharkasaur-B", getImage(getCodeBase(), "images/Sharkasaur-B.jpg"));
		monsters[13] = new ImageMap("Sharkasaur-F", getImage(getCodeBase(), "images/Sharkasaur-F.jpg"));

		monsters[14] = new ImageMap("Shockachu-B", getImage(getCodeBase(), "images/Shockachu-B.jpg"));
		monsters[15] = new ImageMap("Shockachu-F", getImage(getCodeBase(), "images/Shockachu-F.jpg"));

		monsters[16] = new ImageMap("Smelltor-B", getImage(getCodeBase(), "images/Smelltor-B.jpg"));
		monsters[17] = new ImageMap("Smelltor-F", getImage(getCodeBase(), "images/Smelltor-F.jpg"));

		monsters[18] = new ImageMap("Voltasaur-B", getImage(getCodeBase(), "images/Voltasaur-B.jpg"));
		monsters[19] = new ImageMap("Voltasaur-F", getImage(getCodeBase(), "images/Voltasaur-F.jpg"));

		//pf = getImage(getCodeBase(), "images/PlayerFight.gif");
		//pef = getImage(getCodeBase(), "images/PersonFight.gif");
		//tef = getImage(getCodeBase(), "images/TrainerFight.gif");
		splash = getImage(getCodeBase(), "images/Splash.jpg");
		gover = getImage(getCodeBase(), "images/GameOver.jpg");
		sh = getImage(getCodeBase(), "images/Shop.gif");
		gwin = getImage(getCodeBase(), "images/Win.jpg");

	}
	/**
	 * Get the Grid ready to be redisplayed
	 * after it was removed, ie add the Monsters.
	 */
	public void initGrid()
	{
		grid = mp.getGrid();
		grid.addBattleListener(this);
		grid.addRedirectListener(this);
		add(grid, BorderLayout.CENTER);
		grid.setSize(getWidth(), getHeight());
		grid.init();
		grid.setPlayer(pl);
		grid.requestFocus();
	}
	/**
	 * Handle the loading of a Player file.
	 * 
	 * @see Player
	 * @see PlayerLoader
	 */
	public void loadPlayerFile(String file)
	{
		try
		{
			URL url = new URL(getCodeBase(), "maps/" + file + ".pml");
			URLConnection uc = url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader(uc.getInputStream()));
			PlayerLoader pload = new PlayerLoader(in);
			pl = pload.getPlayer();
		}
		catch(Exception ee) { ee.printStackTrace(); }
	}
}