import java.awt.*;
import javax.swing.*;
import java.applet.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * A class to define the Monsters of type Gas
 * handles the leveling up process and naming of
 * attacks.
 * 
 * @see Monster
 * @version 1.2.3.5
 * @author Matt, Sam
 */
public class GasMon extends Monster
{
	/**
	 * The number of levels needed before learning a new attack
	 */
	protected static final int START = 11;
	/**
	 * The level at which all attacks have been learned
	 */
	protected static final int MAX_EARN_ATTACK_LEVEL = 45;
	/**
	 * The name of the first attack
	 */
	protected static final String A1 = "Quick";
	/**
	 * The name of the second attack
	 */
	protected static final String A2 = "Finger-Pulled";
	/**
	 * The name of the third attack
	 */
	protected static final String A3 = "Make-U-Reek";
	/**
	 * The name of the fourth attack
	 */
	protected static final String A4 = "Silent-But-Deadly";
	/**
	 * The name of the fifth attack
	 */
	protected static final String A5 = "Ate-Some-Beans";
	/**
	 * The name of the sixth attack
	 */
	protected static final String A6 = "Mega-Fart";
	/**
	 * The last attack learned (2 <= x <= 5)
	 */
	protected int lastAttack = 2;
	/**
	 * Construct a Gas Monster with the default values:
	 * name is Alligatrix
	 * level is two. 
	 */
	public GasMon()
	{
		type = MapObject.MONSTER;
		attacks = new Vector();
		maxLevel = START;
		name = "Eurekasaur";
		kind = GAS;
		enterable = true;

		level = 2;
		int lev6 = 6*level;
		int lev7 = 7*level;
		int lev10 = 10*level;
		hp = lev7;
		defp = lev10;
		attp = lev6;
		maxHP = lev7;
		maxDef = lev10;
		maxAtt = lev6;
		exp = 0;
		maxExp = lev10;
		addAttack(A1);
		addAttack(A2);
	}

	/**
	 * Constructs a Gas Monster with the name name.
	 * level is two.
	 * 
	 * @param name the name of the Gas Monster
	 */
	public GasMon(String name)
	{
		type = MapObject.MONSTER;
		attacks = new Vector();
		maxLevel = START;
		if(name != null)
			this.name = name;
		else
			name = "Eurekasaur";
		kind = GAS;
		enterable = true;

		level = 2;
		int lev6 = 6*level;
		int lev7 = 7*level;
		int lev10 = 10*level;
		hp = lev7;
		defp = lev10;
		attp = lev6;
		maxHP = lev7;
		maxDef = lev10;
		maxAtt = lev6;
		exp = 0;
		maxExp = lev10;
		addAttack(A1);
		addAttack(A2);
	}

	/**
	 * Empty implemntation unneeded as Monsters are
	 * only displayed in battle
	 */
	public void setImage(Image[] img) { } // Figure out how to make me work
	/**
	 * Sets the max level
	 * always is START + previousMaxLevel
	 * 
	 * @param nml the new Max Level
	 */
	protected void setMaxLevel(byte nml)
	{
		if(nml > maxLevel && nml < MAX_EARN_ATTACK_LEVEL)
			maxLevel = nml;
	}

	/**
	 * Preforms a level up on the Monster, Adds new Attacks
	 * if needed, increases HP, Attack Points, etc.
	 * 
	 * @exception AttacksFullException if this has Monster.MAXATTACKS Attacks
	 * @see Monster
	 * @see Attack
	 */
	protected void levelUp() throws AttacksFullException
	{
		super.levelUp();

		if(level >= maxLevel && level < MAX_EARN_ATTACK_LEVEL)
		{
			setMaxLevel((byte)(maxLevel+START));
			try
			{
				Attack a;
				switch(lastAttack)
				{
					case 2:
						{
						a = new Attack(A3, Attack.DAMAGE3);
						lastAttack++;
						break;
					}
					case 3:
						{
						a = new Attack(A4, Attack.DAMAGE4);
						lastAttack++;
						break;
					}
					case 4:
						{
						a = new Attack(A5, Attack.DAMAGE5);
						lastAttack++;
						break;
					}
					case 5:
						{
						a = new Attack(A6, Attack.DAMAGE6);
						lastAttack++;
					}
					default:
						return;
				}
				if(attacks.size() < MAXATTACKS)
				{
					attacks.add(a);
				}
				else
					throw new AttacksFullException("Attacks are full to the brim remove one to add a new one", a);
			}
			catch(InvalidPowerException e) { } // CANNOT HAPPEN HERE
		}
	}

	/**
	 * Adds the attack specified by its name as parameter att to the Monster
	 * if it has less than Monster.MAXATTACKS
	 * 
	 * @see Monster
	 * @param att the Name of the Attack to add
	 */
	public void addAttack(String att) // Only used for instancitating the grid from a MapLoader
	{
		if(!(getNumberOfAttacks() < Monster.MAXATTACKS))
			return;
		Attack at;
		for(int i = 0; i < attacks.size(); i++)
		{
			at = (Attack)attacks.get(i);
			if(at.getName().equalsIgnoreCase(att))
				return;
		}
		try
		{
			if(att.equalsIgnoreCase(A1))
			{
				attacks.add(new Attack(A1, Attack.DAMAGE1));
				lastAttack = 1;
			}
			else if(att.equalsIgnoreCase(A2))
			{
				attacks.add(new Attack(A2, Attack.DAMAGE2));
				lastAttack = 2;
			}
			else if(att.equalsIgnoreCase(A3))
			{
				attacks.add(new Attack(A3, Attack.DAMAGE3));
				lastAttack = 3;
			}
			else if(att.equalsIgnoreCase(A4))
			{
				attacks.add(new Attack(A4, Attack.DAMAGE4));
				lastAttack = 4;
			}
			else if(att.equalsIgnoreCase(A5))
			{
				attacks.add(new Attack(A5, Attack.DAMAGE5));
				lastAttack = 5;
			}
			else if(att.equalsIgnoreCase(A6))
			{
				attacks.add(new Attack(A6, Attack.DAMAGE6));
				lastAttack = 6;
			}
		}
		catch(InvalidPowerException e) { } // CANNOT HAPPEN
	}

	/**
	 * Removes an Attack and adds a new Attack to the end.
	 * 
	 * @param removed the Attack to remove
	 * @param added the Attack to add
	 * @return true if could remove the Attack and add the new one
	 */
	public boolean substituteAttack(Attack removed, Attack added)
	{
		if(attacks.remove(removed))
		{
			attacks.add(added);
			return true;
		}
		else
			return false;
	}

	/**
	 * Returns the name of Attack A1
	 * 
	 * @return A1
	 */
	public String getA1() { return A1; }

	/**
	 * Returns the name of Attack A2
	 * 
	 * @return A2
	 */
	public String getA2() { return A2; }

	/**
	 * Returns the name of Attack A3
	 * 
	 * @return A3
	 */
	public String getA3() { return A3; }

	/**
	 * Returns the name of Attack A4
	 * 
	 * @return A4
	 */
	public String getA4() { return A4; }

	/**
	 * Returns the name of Attack A5
	 * 
	 * @return A5
	 */
	public String getA5() { return A5; }

	/**
	 * Returns the name of Attack A6
	 * 
	 * @return A6
	 */
	public String getA6() { return A6; }

	/**
	 * Returns a new Gas Monster with
	 * the same name as this.
	 */
	public MapObject getClone() { return new GasMon(name); }
}