import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Abstract class to represent an Item.
 * 
 * @see MonsterBallItem
 * @see PotionItem
 * @see People
 */
public abstract class Item
{
	/**
	 * Brand of MonsterBall
	 */
	public static final int MONSTERBALL = 0;
	/**
	 * Brand of Potion
	 */
	public static final int POTION = 3;

	/**
	 * The name of this Item
	 */
	protected String name;
	/**
	 * The number amount of this that the People has.
	 */
	protected int number;
	/**
	 * The brand of Item this is.
	 */
	protected int brand;

	/**
	 * Sets the name of this Item.
	 * 
	 * @param nn the New Name of this.
	 */
	public void setName(String nn)
	{
		if(nn != null)
			name = nn;
	}
	/**
	 * Gets the name of this Item.
	 * 
	 * @return the name of this Item
	 */
	public String getName() { return name; }
	/**
	 * Adds another of this.
	 */
	public void addItem() { number++; }
	/**
	 * Sets the number of this there is.
	 * 
	 * @param ic the new Item Count
	 */
	public void setItemCount(int ic)
	{
		if(ic > 0)
			this.number = ic;
	}
	/**
	 * Gets the number of this there is
	 * 
	 * @return the number of this that are available
	 */
	public int getItemCount() { return number; }
	/**
	 * Uses the item on the Monster Specified
	 * 
	 * @param m the Monster to use the Item on
	 */
	public abstract boolean useItem(Monster m);
	/**
	 * Gets the Brand of this.
	 * 
	 * @return th Brand of this as defined above.
	 */
	public int getBrand() { return brand; }
	/**
	 * Gets the subBrand of this defined in PotionItem and MonsterBallItem
	 * 
	 * @return the sub Brand of this.
	 * @see MonsterBallItem
	 * @see PotionItem
	 */
	public abstract int getSubBrand();
	/**
	 * Determines if there is equality
	 * 
	 * @param o the Object to compare to
	 * @return true iff they represent the same Item
	 */
	public boolean equals(Object o)
	{
		if(o instanceof Item)
			return equals((Item)o);
		return false;
	}
	/**
	 * Determines if there is equality
	 * 
	 * @param i the Item to compare to
	 * @return true iff they represent the same Item, ie have the same Name
	 */
	public boolean equals(Item i)
	{
		return i.getName().equalsIgnoreCase(name);
	}
}