import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.io.*;
/**
 * Class to construct the Grid as specified in a mml file.
 * 
 * @see Game
 * @see Grid
 */
public class MapLoader
{
	/**
	 * The Grid constructing
	 */
	protected Grid g;
	/**
	 * true iff finished Constructing the Grid
	 */
	protected boolean fin;
	/**
	 * The BufferedReader to read from
	 */
	static BufferedReader br;
	/**
	 * Counters
	 */
	int quotes, bracket, element;
	/**
	 * Char to read into
	 */
	char c;
	/**
	 * int to read into
	 */
	int ch;
	/**
	 * StringBuffer to test against
	 */
	StringBuffer s;
	/**
	 * The blocker, default MapObject for the grid
	 */
	MapObject block, def;

	/**
	 * Construct a new MapLoader using the BufferedReader specified.
	 * 
	 * @param br the BufferedReader
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	public MapLoader(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		g = new Grid();
		this.br = br;
		s = new StringBuffer();
		quotes = 0;
		bracket = 0;
		element = 0;
		startReading();
	}
	/**
	 * Sets the BufferedReader used to process the file, and begins processing.
	 * 
	 * @param br the BufferedReader
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	public boolean setFile(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		if(fin && br != null)
		{
			this.br = br;
			startReading();
			return true;
		}
		return false;
	}
	/**
	 * Returns true iff c is between A and Z case insensitive
	 * 
	 * @return  true iff c is between A and Z case insensitive
	 */
	protected boolean isChar(int c)
	{
		return ( (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') );
	}
	/**
	 * Get the Grid Constructed
	 * 
	 * @return the Grid Constructed
	 * @see Grid
	 */
	public Grid getGrid() { return g; }
	/**
	 * Gets the deafult MapObject for the GridConstructed
	 * 
	 * @return the deafult MapObject for the GridConstructed
	 */
	public MapObject getDefault() { return def; }
	/**
	 * Starts the processing of the file.
	 * 
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void startReading() throws IOException, InvalidFileFormatException
	{
		
		// Read until first keyword
		s.setLength(0);
		while((ch = br.read()) != ' ' && ch != -1)
		{
			if(isChar(ch))
				s.append((char)ch);
		}
		if(!s.toString().equalsIgnoreCase("map"))
			throw new InvalidFileFormatException("Invalid File Format");
		bracket++;
		s.setLength(0);

		while((ch = br.read()) != -1)
		{
			if(ch == '\n')
				continue;
			else if(ch == '<')
			{
				s.setLength(0);
				bracket++;
				while((c = (char)br.read()) != '>' && c != ' ')
				{
					s.append(c);
				}
				String st = s.toString();
				if(st.equalsIgnoreCase("monstersAllowed"))
				{
					processMonstersAllowed(br);
				}
				else if(st.equalsIgnoreCase("person"))
				{
					processPerson(br, true);
				}
				else if(st.equalsIgnoreCase("trainer"))
				{
					processPerson(br, false);
				}
				else if(st.equalsIgnoreCase("player"))
				{
					processPlayer(br);
				}
				else if(st.equalsIgnoreCase("blocker"))
				{
					processBlocker(br);
				}
				else if(st.equalsIgnoreCase("building"))
				{
					processBuilding(br);
				}
				/*else if(st.equalsIgnoreCase("item"))
				{
					processItemMap(br);
				}*/
				else if(st.equalsIgnoreCase("redirect"))
				{
					processRedirect(br);
				}
				else if(st.indexOf("!") != -1) // Comments
				{
					continue;
				}
				else if(st.equalsIgnoreCase("/map"))
				{
					break;
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
			else if(ch == '>')
				bracket--;
			else if(ch == '=')
			{
				if(s.toString().equalsIgnoreCase("default"))
				{
					s.setLength(0);
					while(br.read() != '\"'); // Skip first "
					while((c = (char)br.read()) != '\"')
					{
						s.append(c);
					}
					try
					{
						if(s.toString().equalsIgnoreCase("floor"))
						{
							def = new Floor();
							for(int i = 0; i < Grid.DEFAULTNUM; i++)
							{
								for(int j = 0; j < Grid.DEFAULTNUM; j++)
								{
									g.setObjectAt(new Floor(), i, j);
								}
							}
						}
						else
						{
							def = new Grass();
							for(int i = 0; i < Grid.DEFAULTNUM; i++)
							{
								for(int j = 0; j < Grid.DEFAULTNUM; j++)
								{
									g.setObjectAt(new Grass(), i, j);
								}
							}
						}
						g.setDefault(def);
					}
					catch(Exception e) { }
					s.setLength(0);
					continue;
				}
				else if(s.toString().equalsIgnoreCase("blocker"))
				{
					s.setLength(0);
					while(br.read() != '\"'); // Skip first "
					while((c = (char)br.read()) != '\"')
					{
						s.append(c);
					}
					if(s.toString().equalsIgnoreCase("Wall"))
					{
						block = new Wall();
					}
					else
					{
						block = new Rock();
					}
					s.setLength(0);
					continue;
				}
			}
			else if(ch == '\"')
			{
				if(quotes > 0)
				{
					quotes--;
				}
				else
				{
					quotes++;
				}
			}
			else if(ch == ' ' && quotes == 0)
				continue;
			else if(isChar(ch))
			{
				s.append((char)ch);
			}
		}
		fin = true;
	}
	/**
	 * Handles the Monsters Allowed
	 * 
	 * @param br the BufferedReader to use
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processMonstersAllowed(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i = 0;
		String st;
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("type"))
					{
						continue;
					}
					else if(s.toString().equalsIgnoreCase("monstersAllowed"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				if(s.toString().equalsIgnoreCase("type"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					bracket++;
					st = s.toString();
					if(st.equalsIgnoreCase("Electric"))
					{
						g.allowMonsterOnMap(Monster.ELECTRIC);
						continue;
					}
					else if(st.equalsIgnoreCase("Earth"))
					{
						g.allowMonsterOnMap(Monster.EARTH);
						continue;
					}
					else if(st.equalsIgnoreCase("Fire"))
					{
						g.allowMonsterOnMap(Monster.FIRE);
						continue;
					}
					else if(st.equalsIgnoreCase("Water"))
					{
						g.allowMonsterOnMap(Monster.WATER);
						continue;
					}
					else if(st.equalsIgnoreCase("Gas"))
					{
						g.allowMonsterOnMap(Monster.GAS);
						continue;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(s.toString().equalsIgnoreCase("level"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					try
					{
						int lev = Integer.parseInt(st);
						g.setMonsterLevel(lev);
					}
					catch(Exception e)
					{ 
							throw new InvalidFileFormatException("Invalid File Format");
					}
				}
			}
		}
	}

	/**
	 * Handle the Player
	 * 	 
	 * @param br the BufferedReader to use
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processPlayer(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i, pi = 3, pj = 5;
		String st;
		Player p = new Player();

		while((i = br.read()) != '\n')
		{
			if(i == 'i' || i == 'I')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pi = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			else if(i == 'j' || i == 'J')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pj = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			if(i == '>')
				break;
		}
		try
		{
			g.setObjectAt(p, pi, pj);
			p.setBehind(def);
		}
		catch(Exception e)
		{
			throw new InvalidFileFormatException("Invalid Format");
		}
		while((i = br.read()) != '>');
	}

	/**
	 * Handle the Person
	 * 	 
	 * @param br the BufferedReader to use
	 * @param person pass true if a Person false if a Trainer
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processPerson(BufferedReader br, boolean person) throws IOException, InvalidFileFormatException
	{
		int i, pi = 3, pj = 3;
		String st;
		People p;
		if(person)
			p = new Person();
		else
			p = new Trainer();

		while((i = br.read()) != '\n')
		{
			if(i == 'i' || i == 'I')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pi = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			else if(i == 'j' || i == 'J')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pj = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			if(i == '>')
				break;
		}
		try
		{
			g.setObjectAt(p, pi, pj);
			p.setBehind(def);
		}
		catch(Exception e)
		{
			throw new InvalidFileFormatException("Invalid Format");
		}

		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("money"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("name"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("Direction"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("badge"))
					{
						continue;
					}
						else if(st.equalsIgnoreCase("dialogue"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("trainer"))
					{
						s.setLength(0);
						return;
					}
					else if(st.equalsIgnoreCase("person"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("money"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int mon = Integer.parseInt(s.toString());
						p.setMoney(mon);
					}
					catch(Exception e) 
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				else if(st.equalsIgnoreCase("name"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					p.setName(s.toString());
				}
				else if(st.equalsIgnoreCase("badge"))
				{
					if(p.getType() == MapObject.TRAINER)
					{
						s.setLength(0);
						while((i = br.read()) != '<')
						{
							s.append((char)i);
						}
						((Trainer)p).setBadge(s.toString());
					}
					else
						continue;
				}
				else if(st.equalsIgnoreCase("direction"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("UP"))
					{
						p.setDirection(Grid.UP);
						continue;
					}
					else if(st.equalsIgnoreCase("DOWN"))
					{
						p.setDirection(Grid.DOWN);
						continue;
					}
					else if(st.equalsIgnoreCase("LEFT"))
					{
						p.setDirection(Grid.LEFT);
						continue;
					}
					else if(st.equalsIgnoreCase("RIGHT"));
					{
						p.setDirection(Grid.RIGHT);
						continue;
					}
				}
				else if(st.equalsIgnoreCase("dialogue"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					p.setDialogue(s.toString());
				}
				else if(st.equalsIgnoreCase("monsterVector"))
				{
					processMonsterVector(br, p);
				}
				else if(st.equalsIgnoreCase("itemVector"))
				{
					processItemVector(br, p);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}
	/**
	 * Handle the Monster Vector
	 * 	 
	 * @param br the BufferedReader to use
	 * @param p the People to add the Monsters to
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processMonsterVector(BufferedReader br, People p) throws IOException, InvalidFileFormatException
	{
		int i = 0;
		String st;
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("monsterVector"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket--;
				if(s.toString().equalsIgnoreCase("monster"))
				{
					Monster m = processMonster(br);
					p.addMonster(m);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}
	/**
	 * Handle the Monster
	 * 	 
	 * @param br the BufferedReader to use
	 * @return the Monster specified in th mml
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected Monster processMonster(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		Monster m = null;
		int i=0, pi, pj, lev= 20;
		String st;
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("kind"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("hp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("defp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("attp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("exp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("level"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("attack"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("monster"))
					{
						s.setLength(0);
						return m;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("hp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int hp = Integer.parseInt(s.toString());
						m.setMaxHP(hp);
						m.setHP(hp);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				else if(st.equalsIgnoreCase("defp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int def = Integer.parseInt(s.toString());
						m.setMaxDefense(def);
						m.setDefense(def);
					}
					catch(Exception e) 
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("attp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int att = Integer.parseInt(s.toString());
						m.setMaxAttack(att);
						m.setAttack(att);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("exp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int exp = Integer.parseInt(s.toString());
						m.setMaxExperience(10*lev);
						m.setExperience(exp);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("level"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						lev = Integer.parseInt(s.toString());
						m.setLevel(lev);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("name"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					m.setName(s.toString());
				}
				else if(st.equalsIgnoreCase("attack"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(m.getNumberOfAttacks() >= Monster.MAXATTACKS)
					{
						continue;
					}
					if(st.equalsIgnoreCase("A1"))
					{
						m.addAttack(m.getA1());
					}
					else if(st.equalsIgnoreCase("A2"))
					{
						m.addAttack(m.getA2());
					}
					else if(st.equalsIgnoreCase("A3"))
					{
						m.addAttack(m.getA3());
					}
					else if(st.equalsIgnoreCase("A4"))
					{
						m.addAttack(m.getA4());
					}
					else if(st.equalsIgnoreCase("A5"))
					{
						m.addAttack(m.getA5());
					}
					else if(st.equalsIgnoreCase("A6"))
					{
						m.addAttack(m.getA6());
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("kind"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("Electric"))
					{
						m = new ElectricMon();
					}
					else if(st.equalsIgnoreCase("EARTH"))
					{
						m = new EarthMon();
					}
					else if(st.equalsIgnoreCase("Fire"))
					{
						m = new FireMon();
					}
					else if(st.equalsIgnoreCase("Water"))
					{
						m = new WaterMon();
					}
					else if(st.equalsIgnoreCase("Gas"))
					{
						m = new GasMon();
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
		return m;
	}

	/**
	 * Handle the Item Vector
	 * 	 
	 * @param br the BufferedReader to use
	 * @param p the People to add the Item to
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processItemVector(BufferedReader br, People p) throws IOException, InvalidFileFormatException
	{
		int i=0;
		String st;
		Item m;
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("itemVector"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket--;
				if(s.toString().equalsIgnoreCase("item"))
				{
					m = processItemForVector(br);
					p.addItem(m);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}
	/**
	 * Handle the Item
	 * 	 
	 * @param br the BufferedReader to use
	 * @return the Item created
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected Item processItemForVector(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i=0, pi, pj;
		String st;
		Item it = null;
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("potion"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("quantity"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("monsterBall"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("item"))
					{
						s.setLength(0);
						return it;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("potion"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("HPUPFULL"))
					{
						it = new PotionItem("HP Up Full", PotionItem.HPUPFULL);
					}
					else if(st.equalsIgnoreCase("HPUPHALF"))
					{
						it = new PotionItem("HP Up Half", PotionItem.HPUPHALF);
					}
					else if(st.equalsIgnoreCase("HPUPTHIRD"))
					{
						it = new PotionItem("HP Up Third", PotionItem.HPUPTHIRD);
					}
					else if(st.equalsIgnoreCase("DEFUP"))
					{
						it = new PotionItem("Defense Up", PotionItem.DEFUP);
					}
					else if(st.equalsIgnoreCase("ATTUP"))
					{
						it = new PotionItem("Attack Up", PotionItem.ATTUP);
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("quantity"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					try
					{
						int q = Integer.parseInt(st);
						it.setItemCount(q);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("monsterBall"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("NORMAL"))
					{
						System.out.println("Better");
						it = new MonsterBallItem("Normal MonsterBall", MonsterBallItem.NORMAL);
					}
					else if(st.equalsIgnoreCase("MASTER"))
					{
						it = new MonsterBallItem("Master MonsterBall", MonsterBallItem.MASTER);
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
		return it;
	}

	/**
	 * Handle the Blocker
	 * 	 
	 * @param br the BufferedReader to use
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processBlocker(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i, pi = 3, pj = 0;
		String st;
		MapObject p;
		if(block.getType() == MapObject.ROCK)
			p = new Rock();
		else
			p = new Wall();

		while((i = br.read()) != '\n')
		{
			if(i == 'i' || i == 'I')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pi = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			else if(i == 'j' || i == 'J')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pj = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			if(i == '>')
				break;
		}
		try
		{
			g.setObjectAt(p, pi, pj);
		}
		catch(Exception e)
		{
			throw new InvalidFileFormatException("Invalid Format");
		}
		while((i = br.read()) != '>');
	}
	/**
	 * Handle the Building
	 * 	 
	 * @param br the BufferedReader to use
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processBuilding(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i, pi = 8, pj = 8;
		String st;
		while((i = br.read()) != '\n')
		{
			if(i == 'i' || i == 'I')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pi = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			else if(i == 'j' || i == 'J')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pj = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			if(i == '>')
				break;
		}
		try
		{
			g.setObjectAt(new Building(), pi, pj);
		}
		catch(Exception e)
		{
			throw new InvalidFileFormatException("Invalid Format");
		}
		while((i = br.read()) != '>');
	}
	/**
	 * Handle the Redirect
	 * 	 
	 * @param br the BufferedReader to use
	 * @exception InvalidFileFormatException iff the file is not properly formatted
	 */
	protected void processRedirect(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i, pi = 2, pj = 2;
		String st;
		Redirect r = new Redirect();

		while((i = br.read()) != '\n')
		{
			if(i == 'i' || i == 'I')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pi = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			else if(i == 'j' || i == 'J')
			{
				while((i = br.read()) != '=');
				s.setLength(0);
				while((i = br.read()) != ' ')
				{
					if(i == '>')
						break;
					else
					{
						s.append((char)i);
					}
				}
				try
				{
					pj = Integer.parseInt(s.toString());
				}
				catch(Exception e)
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
			if(i == '>')
				break;
		}
		try
		{
			g.setObjectAt(r, pi, pj);
		}
		catch(Exception e)
		{
			throw new InvalidFileFormatException("Invalid Format");
		}
		while(i!=-1)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("to"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("requires"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("redirect"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("to"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					r.setMapTo(s.toString());
					r.setBehind(def);
				}
				else if(st.equalsIgnoreCase("requires"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						r.setNumberBadgesRequired(Integer.parseInt(s.toString()));
					}
					catch(Exception e)
					{
						throw new InvalidFileFormatException("Invalid Format");
					}
					r.setBehind(def);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid Format");
				}
			}
		}
	}
}