import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
/**
 * Super class of all Objects on the Grid
 * 
 * @see Grid
 */
public abstract class MapObject extends Component
{
	/**
	 * Type Grass
	 */
	public static final int GRASS = 0;
	/**
	 * Type Person
	 */
	public static final int PERSON = 1;
	/**
	 * Type Player
	 */
	public static final int PLAYER = 2;
	/**
	 * Type Rock
	 */
	public static final int ROCK = 3;
	/**
	 * Type Monster
	 */
	public static final int MONSTER = 5;
	/**
	 * Type Building
	 */
	public static final int BUILDING = 6;
	/**
	 * Type Floor
	 */
	public static final int FLOOR = 7;
	/**
	 * Type Redirect
	 */
	public static final int REDIRECT = 8;
	/**
	 * Type Wall
	 */
	public static final int WALL = 9;
	/**
	 * Type Trainer
	 */
	public static final int TRAINER = 11;

	/**
	 * true iff can be entered, ie Grass, Floor
	 */
    protected boolean enterable;
	/**
	 * The type of this
	 */
	protected int type;

	/**
	 * return a clone of this
	 * 
	 * @return a clone of this
	 */
	public abstract MapObject getClone();
	/**
	 * Sets the image to display.
	 * Needs be called before paint
	 * 
	 * @param img the array of images to use
	 */
	public abstract void setImage(Image[] img);
	/**
	 * return true if can be entered, ie Grass, Floor.
	 * 
	 * @return true if can be entered, ie Grass, Floor.
	 */
	public boolean canBeEntered() { return enterable; }
	/**
	 * Gets the type associated with this.
	 * 
	 * @return the type associated with this.
	 */
	public int getType() { return type; }
}