import java.awt.*;
import javax.swing.*;
import java.applet.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Class to define a Monster and its attributes.
 * 
 * @see FireMon
 * @see EarthMon
 * @see WaterMon
 * @see GasMon
 * @see ElectricMon
 */
public abstract class Monster extends MapObject
{
	// The Kinds of Monsters
	/**
	 * Electric kind
	 */
	public static final int ELECTRIC = 0;
	/**
	 * Earth kind
	 */
	public static final int EARTH = 1;
	/**
	 * Fire kind
	 */
	public static final int FIRE = 2;
	/**
	 * Water kind
	 */
	public static final int WATER = 3;
	/**
	 * Gas kind
	 */
	public static final int GAS = 4;

	/**
	 * The Maximum level allowed for Monsters to have
	 */
	public static final int MAXLEVEL = 100;
	/**
	 * The Maximum number of attack to have
	 */
	public static final int MAXATTACKS = 4;

	/**
	 * The HP Of this
	 */
	protected int hp; // Hit Points (life left)
	/**
	 * The Defense Of this
	 */
	protected int defp; //Defense Points
	/**
	 * The Attack Of this
	 */
	protected int attp; // Attack Points
	/**
	 * The Experience Of this
	 */
    protected int exp; // Experience Points
	/**
	 * The Maximum HP Of this, ie full health
	 */
	protected int maxHP; // Max HP for level at
	/**
	 * The Maximum Defense Of this at this level
	 */
	protected int maxDef; // Max def for level at
	/**
	 * The Maximum Attack Of this at this level
	 */
	protected int maxAtt; // Max att for level at
	/**
	 * The Maximum Experience before a level up
	 */
	protected int maxExp; // Max exp before Level up
	/**
	 * The level of this
	 */
	protected int level; // Level
	/**
	 * The Maximum level before learning a new attack
	 */
	protected int maxLevel; // Max level before learn new Attack
	/**
	 * The kind of Monster this is
	 */
	protected int kind; // Kind of Monster
	/**
	 * The attacks this knows
	 */
	protected Vector attacks; // Attacks I know
	/**
	 * The name of this
	 */
	protected String name;
	/**
	 * tru iff battling
	 */
	protected boolean battle;
	/**
	 * The MapObjct to display Over
	 */
	static MapObject mp;

	/**
	 * Learn the attack specifid by attack
	 * 
	 * @param attack The Attack to learn
	 */
	public abstract void addAttack(String attack);
	/**
	 * Gets the name of the first Attack
	 * 
	 * @return the name of the first Attack
	 */
	public abstract String getA1();
	/**
	 * Gets the name of the second Attack
	 * 
	 * @return the name of the second Attack
	 */
	public abstract String getA2();
	/**
	 * Gets the name of the third Attack
	 * 
	 * @return the name of the third Attack
	 */
	public abstract String getA3();
	/**
	 * Gets the name of the fourth Attack
	 * 
	 * @return the name of the fourth Attack
	 */
	public abstract String getA4();
	/**
	 * Gets the name of the fifth Attack
	 * 
	 * @return the name of the fifth Attack
	 */
	public abstract String getA5();
	/**
	 * Gets the name of the sixth Attack
	 * 
	 * @return the name of the sixth Attack
	 */
	public abstract String getA6();
	/**
	 * Substitute one Attack for another.
	 * 
	 * @param removed the Attack to remove
	 * @param added the Attack to add
	 */
	public abstract boolean substituteAttack(Attack removed, Attack added);
	/**
	 * Sets the hp
	 * 
	 * @param the new HP of this
	 */
	public void setHP(int nhp)
	{
		if(nhp < 0)
			hp = 0;
		else if(nhp <= maxHP)
			hp = nhp;
		else
			hp = maxHP;
	}
	/**
	 * Gets the HP of this
	 * 
	 * @return the HP of this
	 */
	public int getHP() { return hp; }
	/**
	 * Sets the maximum hp
	 * 
	 * @param the new maximum HP of this
	 */
	protected void setMaxHP(int nmhp)
	{
		if(nmhp > maxHP && level <= MAXLEVEL)
			maxHP = nmhp;
	}
	/**
	 * Gets the maximum hp
	 * 
	 * @return the new maximum HP of this
	 */
	public int getMaxHP() { return maxHP; }
	/**
	 * Sets the defense of this
	 * 
	 * @param ndp the new defense of this
	 */
	public void setDefense(int ndp)
	{
		if(ndp <= 0)
			return;
		if(ndp <= maxDef)
			defp = ndp;
		else
			defp = maxDef;
	}
	/**
	 * Gers the defense of this
	 * 
	 * @return the defense of this
	 */
	public int getDefense() { return defp; }
	/**
	 * Sets the Maximum defense of this
	 * 
	 * @param nmdp the new Maximum Defense of this
	 */
	protected void setMaxDefense(int nmdp)
	{
		if(nmdp > maxDef && level <= MAXLEVEL)
			maxDef = nmdp;
	}
	/**
	 * Gets the Maximum Defense
	 * 
	 * @return the Maximum defense of this
	 */
	public int getMaxDefense() { return maxDef; }
	/**
	 * Sets the Attack points of this
	 * 
	 * @param nap the new Attack Points
	 */
	public void setAttack(int nap)
	{
		if(nap <= 0)
			return;
		if(nap <= maxAtt)
			attp = nap;
		else
			attp = maxAtt;
	}
	/**
	 * Gets the Attack Points of this.
	 * 
	 * @return the Attack Points of this
	 */
	public int getAttack() { return attp; }
	/**
	 * Sets the Maximum Attack Points of this
	 * 
	 * @param nmap the new Maximum Attack Points of this
	 */
	protected void setMaxAttack(int nmap)
	{
		if(nmap > maxAtt && level <= MAXLEVEL)
			maxAtt = nmap;
	}
	/**
	 * Gets the Maximum Attack Points of this
	 * 
	 * @return the Maximum Attack Points of this
	 */
	public int getMaxAttack() { return maxAtt; }
	/**
	 * Sets the Experience Pints and levels up if needed.
	 * 
	 * @param nep the new Experience Points of this
	 * @return true iff leveled up
	 */
	protected boolean setExperience(int nep) throws AttacksFullException
	{
		if(nep > exp && nep <= maxExp)
		{
			exp = nep;
			return false;
		}
		else if(nep > maxExp && level <= MAXLEVEL)
		{
			levelUp();
			return true;
		}
		return false;
	}
	/**
	 * Gets the Experience of this
	 * 
	 * @return the Experience of this
	 */
	public int getExperience() { return exp; }

	/**
	 * Sets the maximum Experience of this, the amount required before a level up
	 * 
	 * @param nmep the New Maximum Experience Ponts
	 */
	protected void setMaxExperience(int nmep)
	{
		if(nmep > maxExp && level <= MAXLEVEL)
			maxExp = nmep;
	}
	/**
	 * Gets the Maximum Experience Pointsup
	 * 
	 * @return the Maximum Experience Points, the amount required before a level up
	 */
	public int getMaxExperience() { return maxExp; }
	/**
	 * Gets the level of this
	 * 
	 * @return the level of this
	 */
	public int getLevel() { return level; }
	/**
	 * Gets the Kind of this
	 * 
	 * @return the kind of this
	 */
	public int getKind() { return kind; }
	/**
	 * Sets the level of this.
	 * 
	 * @param level the level to mak this
	 */
	public void setLevel(int level) // Only used by MapLoader
	{
		if(level <= MAXLEVEL)
			this.level = level;
	}
	/**
	 * Gets the Max level
	 * 
	 * @return the max level before a new attack is learned
	 */
	public int getMaxLevel() { return maxLevel; }
	/**
	 * Gets the number of attacks this knows.
	 */
	public int getNumberOfAttacks() { return attacks.size(); }
	/**
	 * Preforms a level up increasing attack, defense, and hp.
	 * 
	 * @exception AttacksFullException iff this has more than Monster.MAXATTACKS
	 */
	protected void levelUp() throws AttacksFullException
	{
		if((level == MAXLEVEL))
			return;
		level++;

		int maxh = 0, maxa = 0, maxd = 0;

		setMaxExperience(10*level);

		switch(getKind())
		{
			case ELECTRIC:
			{
				maxa = maxAtt + 10;
				maxd = maxDef + 7;
				maxh = maxHP + 6;
				break;
			}
			case EARTH:
			{
				maxa = maxAtt + 7;
				maxd = maxDef + 6;
				maxh = maxHP + 10;
				break;
			}
			case FIRE:
			{
				maxa = maxAtt + 6;
				maxd = maxDef + 7;
				maxh = maxHP + 10;
				break;
			}
			case WATER:
			{
				maxa = maxAtt + 7;
				maxd = maxDef + 10;
				maxh = maxHP + 6;
				break;
			}
			case GAS:
			{
				maxa = maxAtt + 6;
				maxd = maxDef + 10;
				maxh = maxHP + 7;
				break;
			}
		}
		setMaxAttack(maxa);
		setMaxDefense(maxd);
		setMaxHP(maxh);
		setMaxExperience(maxExp+10);

		// Make sure any amount missing is missing after level up
		setAttack((attp - maxAtt) + maxa);
		setDefense((defp - maxDef) + maxd);
		setHP((hp - maxHP) + maxh);
	}

	/**
	 * Finish the fight with the opposing Monster, ie give this the requisite experience
	 * 
	 * @param m the Opposing Monster
	 * @exception AttacksFullException iff this has more than Monster.MAXATTACKS, and is leveling up
	 */
	public boolean finishFight(Monster m) throws AttacksFullException
	{
		int nmep = (2 * m.getLevel()) - this.level;
		boolean b = false;
		if(level < 10)
		{
			if(nmep > 0)
			{
				if(setExperience(exp+nmep))
					b = true;
			}
			else
			{
				if(setExperience(exp + 2))
					b = true;
			}
		}
		else if(level < 50)
		{
			if(nmep > 15)
			{
				if(setExperience(exp+nmep))
					b = true;
			}
			else
			{
				if(setExperience(exp + 15))
					b = true;
			}
		}
		else
		{
			if(nmep > 40)
			{
				if(setExperience(exp+nmep))
					b = true;
			}
			else
			{
				if(setExperience(exp + 40))
					b = true;
			}
		}
		return b;
	}
	/**
	 * Sets the name of this Monster
	 * 
	 * @param name The Name of this
	 */
	public void setName(String name)
	{
		if(name != null)
			this.name = name;
	}
	/**
	 * Gets the name of this.
	 * 
	 * @return the name of this
	 */
	public String getName() { return name; }
	/**
	 * Gets the array of attacks available to this, adds null if less then Monster.MAXATTACKS
	 * 
	 * @return the array of attacks available to this, adds null if less then Monster.MAXATTACKS
	 */
	public String[] getAttacks()
	{
		String[] s = new String[MAXATTACKS];
		for(int i = 0; i < attacks.size(); i++)
		{
			s[i] = ((Attack)attacks.get(i)).getName();
		}
		return s;
	}
	/**
	 * Gets the Attack known at the specified position
	 * 
	 * @param pos the index of the Attack to return
	 * @return the Attack at the specified position 
	 */
	public Attack getAttack(int pos)
	{
		if(pos >= 0 && pos < attacks.size())
		{
			return (Attack)attacks.get(pos);
		}
		return null;
	}
	/**
	 * Tests for equality
	 * 
	 * @param o the Object to compare to 
	 * @return true iff equivalant
	 */
	public boolean equals(Object o)
	{
		if(o instanceof Monster)
			return equals((Monster)o);
		return false;
	}
	/**
	 * Tests for equality, ie if all the parameters are the same
	 * 
	 * @param m the Monster to compare to 
	 * @return true iff equivalant
	 */
	public boolean equals(Monster m)
	{
		if(m.getName().equals(name) && m.getHP() == hp && m.getDefense() == defp && m.getAttack() == attp && m.getExperience() == exp && m.getLevel() == level
			&& m.getKind() == kind && m.getMaxHP() == maxHP && m.getMaxDefense() == maxDef && m.getMaxAttack() == maxAtt && m.getMaxExperience() == maxExp
			&& m.getMaxLevel() == maxLevel)
			return true;
		return false;
	}
	/**
	 * Returns true iff battling
	 * 
	 * @return true iff battling
	 */
	public boolean isBattling() { return battle; }
	/**
	 * Starts a fight
	 */
	public void startBattle() { battle = true; }
	/**
	 * Stop a fight
	 */
	public void stopBattle() { battle = false; }
	/**
	 * Sets the MapObject to paint where this is.
	 * 
	 * @param m the MapObject to paint where this is
	 */
	public void setBehind(MapObject m)
	{
		if(m == null)
			return;
		else if(m.canBeEntered())
			this.mp = m;
	}
	/**
	 * Draws this for the Grid.
	 * 
	 * @param g the Graphics.
	 */
	public void paint(Graphics g)
	{
		if(mp != null)
			mp.paint(g);
	}
}