import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Class to define a NPC, carries Monsters, and Items.
 * 
 * @see People
 * @author Matt
 */
public class Person extends People
{
	/**
	 * The four Images to paint corresponding to the four directions
	 */
	protected static Image img, bimg, limg, rimg; // The Four directions
	/**
	 * The AI to control movement around the Grid
	 * 
	 * @see PersonAI
	 * @see Grid
	 */
	protected PersonAI ai;
	/**
	 * Construct a default Player,
	 * Name of NPC
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on Grass
	 * Facing up
	 * 
	 * @see Item
	 * @see Monster
	 */
	public Person()
	{
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "NPC";
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		activeMon = 0;
		dlg = "";
	}
	/**
	 * Construct a default Player,
	 * Name of NPC
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on Grass
	 * Facing as specified
	 * 
	 * @param dir the direction facing as specified in Grid
	 * @see Grid
	 * @see Item
	 * @see Monster
	 */
	public Person(int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "NPC";
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		activeMon = 0;
		dlg = "";
	}
	/**
	 * Construct a default Player,
	 * Name of NPC
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on MapObject specified
	 * Facing up
	 * 
	 * @param behind the MapObject to stand on
	 * @see Item
	 * @see Monster
	 */
	public Person(MapObject behind)
	{
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		name = "NPC";
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name of NPC
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on MapObject specified
	 * Facing as specified
	 * 
	 * @param behind the MapObject to stand on
	 * @param dir the direction facing as specified in Grid
	 * @see Item
	 * @see Monster
	 */
	public Person(MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		name = "NPC";
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name as specified
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on Grass
	 * Facing up
	 * 
	 * @param name the Name for this, needs to be unique for a Grid
	 * @see Grid
	 * @see Item
	 * @see Monster
	 */
	public Person(String name)
	{
		this.dir = Grid.DOWN;
		this.behind = new Grass();
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		this.name = name;
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name as specified
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * No Dialogue
	 * Standing on MapObject specified
	 * Facing as specified
	 * 
	 * @param name the Name for this, needs to be unique for a Grid
	 * @param behind the MapObject to stand on
	 * @param dir the direction facing as specified in Grid
	 * @see Grid
	 * @see Item
	 * @see Monster
	 */
	public Person(String name, MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PERSON;
		activeMon = 0;
		money = 0;
		this.name = "name";
		ai = new PersonAI(this);
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}

	/**
	 * Sets the Image for this.
	 * Needs to be called before calls to paint
	 * 
	 * @param img the array of Image to use, needs be length 4.
	 */
	public void setImage(Image[] img)
	{
		if(img.length == 4)
		{
			this.img = img[0];
			this.bimg = img[1];
			this.limg = img[2];
			this.rimg = img[3];
		}
	}
	/**
	 * Sets th Grid for the PersonAI to use, and initiates the PersonAI.
	 * 
	 * @param g the Grid to pass to PersonAI
	 * @see Grid
	 * @see PersonAI
	 */
	public void setGrid(Grid g)
	{
		ai.setGrid(g);
	}
	/**
	 * Returns a clone of this having the same name, behind and direction facing.
	 * 
	 * @return a clone of this having the same name, behind and direction facing.
	 */
	public MapObject getClone() { return new Person(name, behind, dir); }
	/**
	 * Draws the image for this based upon the direction.
	 * 
	 * @param g the Graphics
	 */
	public void paint(Graphics g)
	{
		behind.paint(g);

		int wi, he;
		switch(dir)
		{
			case Grid.LEFT:
			{
				g.drawImage(limg, 0, 0, this);
				break;
			}
			case Grid.RIGHT:
			{
				g.drawImage(rimg, 0, 0, this);
				break;
			}
			case Grid.UP:
			{
				g.drawImage(bimg, 0, 0, this);
				break;
			}
			default:
			{
				g.drawImage(img, 0, 0, this);
				break;
			}
		}
	}
}