import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Class used to move a NPC around the Grid.
 * 
 * @see People
 * @see Person
 * @see Trainer
 * @see Grid
 */
public class PersonAI implements Runnable
{
	/**
	 * The name of the last person fought
	 */
	private String lastPerson = "";
	/**
	 * The grid
	 */
	private Grid grid;
	/**
	 * The person moving
	 */
	private Person p;
	/**
	 * RNG
	 */
	private Random r = new Random(System.currentTimeMillis());
	/**
	 * My Row
	 */
	private int pi; // ROW
	/**
	 * My Column
	 */
	private int pj; // COL
	/**
	 * The Number of Battles, and th Number of Moves
	 */
	private int numBattles, numMoves; // Needed to prevent livelock with n-People gathered in a square
	/**
	 * How far to be able to see in the direction I am facing
	 */
	protected static final int los = 5;
	/**
	 * My Thread
	 */
	private Thread t;


	/**
	 * Tie this to a Person to control.
	 * 
	 * @param p the Person to control
	 */
	public PersonAI(Person p)
	{
		this.p = p;
		numBattles = 0;
		numMoves = 0;
	}

	/**
	 * Get the people near my Location, one space any direction.
	 * 
	 * @param i the row
	 * @param j the column
	 */
	private MapObject[] peopleNextTo(int i, int j)
	{
		MapObject mp[] = new MapObject[9], mptemp;
		int k = 0;
		for (int x = pi - 1; x <= pi + 1; x++) 
		{
			for (int y = pj - 1; y <= pj + 1; y++) 
			{
				if (x != pi || y != pj) 
				{
					if ( grid.isValidLocation(x, y) )
					{
						try
						{
							mptemp = grid.mapObjectAt(x, y);
							if(mptemp.getType() == MapObject.PERSON || mptemp.getType() == MapObject.PLAYER)
							{
								if( !( ((People)mptemp).getName().equals(p.getName()) ) && !( ((People)mptemp).isBattling() ) )
								{
									mp[k] = mptemp;
									k++;
								}
							}
						}
						catch(InvalidLocationException e) { }
						catch(EmptyLocationException e) { } // Normal to have happen
					}
				}
			}
		}
		if(k != 0)
			return mp;
		return null;
	}
	/**
	 * Move the Person in the direction specified.
	 * 
	 * @param dir the Direction to move
	 */
	protected void move(int dir) throws NonEmptyException, InvalidLocationException, EmptyLocationException
	{
		numBattles = 0;
		numMoves++;
		if(dir == grid.LEFT) // Left
		{
			if(grid.canMove(pi, pj-1))
			{
				p.setDirection(grid.LEFT);
				grid.moveMapObjectTo(pi, pj, pi, pj-1);
				pj--;
				return;
			}
		} 
		else if(dir == grid.RIGHT) // Right
		{
			if(grid.canMove(pi, pj+1))
			{
				p.setDirection(grid.RIGHT);
				grid.moveMapObjectTo(pi, pj, pi, pj+1);
				pj++;
				return;
			}
		}
		else if(dir == grid.UP) // Up
		{
			if(grid.canMove(pi-1, pj))
			{
				p.setDirection(grid.UP);
				grid.moveMapObjectTo(pi, pj, pi-1, pj);
				pi--;
				return;
			}
		}
		else // Down
		{
			if(grid.canMove(pi+1, pj))
			{
				p.setDirection(grid.DOWN);
				grid.moveMapObjectTo(pi, pj, pi+1, pj);
				pi++;
				return;
			}
		}
		if(grid.canMove(pi, pj-1) || grid.canMove(pi, pj+1)|| grid.canMove(pi-1, pj) || grid.canMove(pi+1, pj))
				 move(r.nextInt(4));
		else
			return;
	}
	/**
	 * Handle the actual Moving, based on a RNG
	 */
	protected void moveMe()
	{
		if(p.isBattling())
			return;
		try
		{
			MapObject[] mp = peopleNextTo(pi, pj);
			int dir = r.nextInt(4);
			boolean b = false;
			int k = 0;
			String advis = "";
			MapObject qr = grid.isPeopleInDirection(pi, pj, p.getDirection(), los);

			if(mp != null)
			{
				int i = 0;
				for(i = 0; i < mp.length && mp[i] != null; i++)
				{
					advis = ((People)mp[i]).getName();
					if(!(advis.equals(lastPerson)) || mp[i].getType() == MapObject.PLAYER)
					{
						b = true;
						break;
					}
				}
				if(b && numBattles == 0 && numMoves >= 4)
				{
					synchronized(mp[i])
					{
						numBattles++;
						numMoves = 0;
						if(((People)mp[i]).isBattling())
							return;
						p.battle();
						if(mp[i].getType() == MapObject.PLAYER)
						{
							grid.beginBattleWithPlayer(p);
							lastPerson = "";
						}
						else
						{
							lastPerson = advis;
							((Person)mp[i]).battle();
						}
						return;
					}
				}
			}
			else if(qr != null)
			{
				if( !( ((People)qr).getName().equals(lastPerson) ) )
				{
					move(p.getDirection()); // Move toward if not lastPerson
					return;
				}
			}
			move(dir);
		}
		catch(Exception e) { }
	}
	/**
	 * Set the Grid, set to null to kill the thread.
	 * 
	 * @param the Grid to move on, set to null to kill this thread
	 * @see Grid
	 */
	public void setGrid(Grid g)
	{
		boolean b = false;
		if(this.grid != null)
			b = true;
		this.grid = g;
		try
		{
			Location l = grid.locationOf(p);
			pi = l.getI();
			pj = l.getJ();
			if(!b)
			{
				t = new Thread(this);
				t.start();
			}
		}
		catch(Exception e) { }
	}
	/**
	 * Move this around the Grid, and sleep a random
	 * amount of time.
	 */
	public void run()
	{
		while(grid != null && !p.isBeaten())
		{
			try
			{
				int time = 0;
				synchronized(grid)
				{
					if(!p.isBattling())
					{
						moveMe();
						int count = 0;
						do
						{
							time = (int)(3000.*Math.random());
							count++;
						} while(time < 1500 && count < 5);
						if(count >= 4)
							time = 1500;
					}
					else
					{
						time = 6000;
						p.stopBattle();
					}
					grid.repaint(pi, pj);
				}
				Thread.sleep(time); // Wait rand time bt 1.5 and 3 sec exclusive
			}
			catch(Exception e) { }
		}
	}
}