import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Class to define a Player, carries Monsters, Items and Badges.
 * 
 * @see People
 * @author Matt
 */
public class Player extends People
{
	/**
	 * The four Images to paint corresponding to the four directions
	 */
	protected static Image img, bimg, limg, rimg; // The Four directions
	/**
	 * The badges acquired
	 */
	Vector badges;

	/**
	 * Construct a default Player,
	 * Name of Player1
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on Grass
	 * Facing up
	 * 
	 * @see Item
	 * @see Monster
	 */
	public Player()
	{
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "Player1";
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name of Player1
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on Grass
	 * Facing dir
	 * 
	 * @param dir the direction to face, specified in Grid
	 * @see Item
	 * @see Monster
	 * @see Grid
	 */
	public Player(int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "Player1";
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name of Player1
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on MapObject specified
	 * Facing up
	 * 
	 * @param behind the MapObjct to be standing on
	 * @see Item
	 * @see Monster
	 */
	public Player(MapObject behind)
	{
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		name = "Player1";
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name of Player1
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on the MapObject specifid
	 * Facing the direction specified
	 * 
	 * @param behind the MapObject to stand on
	 * @param dir the direction to face as specified in Grid
	 * @see Item
	 * @see Monster
	 * @see Grid
	 */
	public Player(MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		name = "Player1";
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Construct a Player,
	 * Name as specified
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on Grass
	 * Facing up
	 * 
	 * @param name the name of this Player, non null
	 * @see Item
	 * @see Monster
	 */
	public Player(String name)
	{
		this.dir = Grid.DOWN;
		this.behind = new Grass();
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		this.name = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Construct a default Player,
	 * Name as specified
	 * No Money
	 * No Monsters
	 * No Items
	 * No Badges
	 * Standing on as specified
	 * Facing as specified
	 * 
	 * @param name the Name of this
	 * @param behind the MapObject standing on
	 * @param dir the Direction facing as specified in Grid
	 * @see Item
	 * @see Monster
	 * @see Grid
	 */
	public Player(String name, MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.PLAYER;
		activeMon = 0;
		money = 0;
		this.name = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		badges = new Vector();
	}
	/**
	 * Sets the Image for this.
	 * Needs to be called before calls to paint
	 * 
	 * @param img the array of Image to use, needs be length 4.
	 */
	public void setImage(Image[] img)
	{
		if(img.length == 4)
		{
			this.img = img[0];
			this.bimg = img[1];
			this.limg = img[2];
			this.rimg = img[3];
		}
	}
	/**
	 * Determines if the Player has the badge already.
	 * 
	 * @return true if the Player has the badge already, else false
	 * @see Trainer
	 */
	public boolean hasBadge(String badge)
	{
		if(badge == null)
			return false;
		for(int i = 0; i < badges.size(); i++)
		{
			if(((String)badges.get(i)).equalsIgnoreCase(badge))
			{
				System.out.println((String)badges.get(i) + " || " + badge);
				return true;
			}
		}
		return false;
	}
	/**
	 * Adds the badge to the Player.
	 * 
	 * @param badge the name of the Badge to add
	 * @see Trainer
	 */
	public void addBadge(String badge)
	{
		if(badge == null)
			return;
		if(!hasBadge(badge))
			badges.add(badge);
	}
	/**
	 * Gets the number of Badges acquired.
	 * 
	 * @return the number of Badges acquired.
	 */
	public int getBadgeCount() { return badges.size(); }
	/**
	 * Returns a default Player as a MapObject
	 * 
	 * @see MapObject
	 */
	public MapObject getClone() { return new Player(); }
	/**
	 * Draws the Player using the Images passed into setImage.
	 * 
	 * @param g the Graphics
	 */
	public void paint(Graphics g)
	{
		behind.paint(g);

		int wi, he;
		switch(dir)
		{
			case Grid.LEFT:
			{
				g.drawImage(limg, 0, 0, this);
				break;
			}
			case Grid.RIGHT:
			{
				g.drawImage(rimg, 0, 0, this);
				break;
			}
			case Grid.UP:
			{
				g.drawImage(bimg, 0, 0, this);
				break;
			}
			default:
			{
				g.drawImage(img, 0, 0, this);
				break;
			}
		}
	}
}