import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.io.*;
/**
 * Class to load a Player from a pml file.  This is based closely upon mml.
 * 
 * @see MapLoader
 * @see Player
 * @author Matt
 */
public class PlayerLoader
{
	/**
	 * The Player creating
	 */
	Player p;
	/**
	 * Finished marker
	 */
	protected boolean fin;
	/**
	 * Read from the BufferedReader
	 */
	static BufferedReader br;
	/**
	 * Counters
	 */
	int quotes, bracket, element;
	/**
	 * Read characters
	 */
	char c;
	int ch, i;
	/**
	 * Store values in
	 */
	StringBuffer s;

	/**
	 * Temporary storage
	 */
	StringBuffer sbtemp = new StringBuffer();

	/**
	 * Construct a new PlayerLoader using the BufferedReader specified.
	 * 
	 * @param br the BufferedReader to load from
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 */
	public PlayerLoader(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		p = new Player();
		this.br = br;
		s = new StringBuffer();
		quotes = 0;
		bracket = 0;
		element = 0;
		startReading();
	}
	/**
	 * Gets the Player created
	 * 
	 * @return the Player created
	 * @see Player
	 */
	public Player getPlayer() { return p; }
	/**
	 * Returns true iff is between A and Z case-insensitive.
	 * 
	 * @param c the charactr to compare
	 * @return true iff is between A and Z case-insensitive.
	 */
	protected boolean isChar(int c)
	{
		return ( (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') );
	}
	/**
	 * Starts reading from the file.
	 * 
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 */
	protected void startReading() throws IOException, InvalidFileFormatException
	{
		
		// Read until first keyword
		s.setLength(0);
		while((ch = br.read()) != '>' && ch != -1)
		{
			if(isChar(ch))
				s.append((char)ch);
		}
		if(!s.toString().equalsIgnoreCase("Player"))
			throw new InvalidFileFormatException("Invalid File Format" + s.toString());
		bracket++;
		s.setLength(0);

		while((ch = br.read()) != -1)
		{
			if(ch == '\n')
				continue;
			else if(ch == '<')
			{
				s.setLength(0);
				bracket++;
				while((c = (char)br.read()) != '>' && c != ' ')
				{
					s.append(c);
				}
				String st = s.toString();
				if(st.equalsIgnoreCase("name"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					p.setName(s.toString());
				}
				else if(st.equalsIgnoreCase("money"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					try
					{
						p.setMoney(Integer.parseInt(st));
					}
					catch(Exception e)
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("monsterVector"))
				{
					s.setLength(0);
					while((i = br.read()) != '=');
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("home"))
					{
						processMonsterVector(br, p, false);
					}
					else if(st.equalsIgnoreCase("hand"))
					{
						processMonsterVector(br, p, true);
					}
					else
						throw new InvalidFileFormatException("Invalid File Format");
				}
				else if(st.equalsIgnoreCase("itemVector"))
				{
					processItemVector(br, p);
				}
				else if(st.equalsIgnoreCase("badgeVector"))
				{
					processBadge(br, p);
				}
				else if(st.indexOf("!") != -1) // Comments
				{
					continue;
				}
				else if(st.equalsIgnoreCase("/player"))
				{
					fin = true;
					continue;
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
			else if(ch == '>')
				bracket--;
		}
		fin = true;
	}
	/**
	 * Handles the Monster Vector of the Player
	 * 
	 * @param br Read from
	 * @param p the Player to add to
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 * @see Player
	 */
	protected void processMonsterVector(BufferedReader br, Player p, boolean hand) throws IOException, InvalidFileFormatException
	{
		int i;
		String st;
		while(true)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("monsterVector"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket--;
				if(s.toString().equalsIgnoreCase("monster"))
				{
					Monster m = processMonster(br);
					if(hand)
						p.addMonster(m);
					else
						p.addMonsterHome(m);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}
	/**
	 * Handles the Monster for the Monster Vector
	 * 
	 * @param br the BufferedReader to read from
	 * @return the Monster loaded
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 * @see Monster
	 */
	protected Monster processMonster(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		Monster m = null;
		int i, pi, pj, lev = 20;
		String st;
		while(true)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("kind"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("hp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("defp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("attp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("exp"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("level"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("attack"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("monster"))
					{
						s.setLength(0);
						return m;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("hp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int hp = Integer.parseInt(s.toString());
						m.setMaxHP(hp);
						m.setHP(hp);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}

				}
				else if(st.equalsIgnoreCase("defp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int def = Integer.parseInt(s.toString());
						m.setMaxDefense(def);
						m.setDefense(def);
					}
					catch(Exception e) 
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("attp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int att = Integer.parseInt(s.toString());
						m.setMaxAttack(att);
						m.setAttack(att);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("exp"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						int exp = Integer.parseInt(s.toString());
						m.setMaxExperience(m.getLevel()*10);
						m.setExperience(exp);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("level"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					try
					{
						lev = Integer.parseInt(s.toString());
						m.setLevel(lev);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("name"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					m.setName(s.toString());
				}
				else if(st.equalsIgnoreCase("attack"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(m.getNumberOfAttacks() >= Monster.MAXATTACKS)
					{
						continue;
					}
					if(st.equalsIgnoreCase("A1"))
					{
						m.addAttack(m.getA1());
					}
					else if(st.equalsIgnoreCase("A2"))
					{
						m.addAttack(m.getA2());
					}
					else if(st.equalsIgnoreCase("A3"))
					{
						m.addAttack(m.getA3());
					}
					else if(st.equalsIgnoreCase("A4"))
					{
						m.addAttack(m.getA4());
					}
					else if(st.equalsIgnoreCase("A5"))
					{
						m.addAttack(m.getA5());
					}
					else if(st.equalsIgnoreCase("A6"))
					{
						m.addAttack(m.getA6());
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("kind"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("Electric"))
					{
						m = new ElectricMon();
					}
					else if(st.equalsIgnoreCase("EARTH"))
					{
						m = new EarthMon();
					}
					else if(st.equalsIgnoreCase("Fire"))
					{
						m = new FireMon();
					}
					else if(st.equalsIgnoreCase("Water"))
					{
						m = new WaterMon();
					}
					else if(st.equalsIgnoreCase("Gas"))
					{
						m = new GasMon();
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}

	/**
	 * Handles the Item Vector of the Player
	 * 
	 * @param br Read from
	 * @param p the Player to add to
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 * @see Player
	 */
	protected void processItemVector(BufferedReader br, Player p) throws IOException, InvalidFileFormatException
	{
		int i;
		String st;
		Item m;
		while(true)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("itemVector"))
					{
						s.setLength(0);
						return;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket--;
				if(s.toString().equalsIgnoreCase("item"))
				{
					m = processItemForVector(br);
					p.addItem(m);
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}
	/**
	 * Handles the Item for the Item Vector
	 * 
	 * @param br the BufferedReader to read from
	 * @return the Item loaded
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 * @see Item
	 */
	protected Item processItemForVector(BufferedReader br) throws IOException, InvalidFileFormatException
	{
		int i, pi, pj;
		String st;
		Item it = null;
		while(true)
		{
			i = br.read();
			if(i == '>')
				bracket--;
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("potion"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("quantity"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("monsterBall"))
					{
						continue;
					}
					else if(st.equalsIgnoreCase("item"))
					{
						s.setLength(0);
						return it;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket++;
				st = s.toString();
				if(st.equalsIgnoreCase("potion"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("HPUPFULL"))
					{
						it = new PotionItem("HP Up Full", PotionItem.HPUPFULL);
					}
					else if(st.equalsIgnoreCase("HPUPHALF"))
					{
						it = new PotionItem("HP Up Half", PotionItem.HPUPHALF);
					}
					else if(st.equalsIgnoreCase("HPUPTHIRD"))
					{
						it = new PotionItem("HP Up Third", PotionItem.HPUPTHIRD);
					}
					else if(st.equalsIgnoreCase("DEFUP"))
					{
						it = new PotionItem("Defense Up", PotionItem.DEFUP);
					}
					else if(st.equalsIgnoreCase("ATTUP"))
					{
						it = new PotionItem("Attack Up", PotionItem.ATTUP);
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format: At " + st);
					}
				}
				else if(st.equalsIgnoreCase("quantity"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					try
					{
						int q = Integer.parseInt(st);
						it.setItemCount(q);
					}
					catch(Exception e) 
					{ 
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else if(st.equalsIgnoreCase("monsterBall"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					st = s.toString();
					if(st.equalsIgnoreCase("NORMAL"))
					{
						it = new MonsterBallItem("Normal MonsterBall", MonsterBallItem.NORMAL);
					}
					else if(st.equalsIgnoreCase("MASTER"))
					{
						it = new MonsterBallItem("Master MonsterBall", MonsterBallItem.MASTER);
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}

	/**
	 * Handles the Badge for the Player
	 * 
	 * @param br the BufferedReader to read from
	 * @param p the Player to add the Badges to
	 * @exception InvalidFileFormatException if the file is not a proper pml document
	 * @see Player
	 * @see Trainer
	 */
	protected void processBadge(BufferedReader br, Player p) throws IOException, InvalidFileFormatException
	{
		int i;
		String st;
		while(true)
		{
			i = br.read();
			if(i == '>')
			{
				bracket--;
			}
			else if(i == '<')
			{
				i = br.read();
				if(i == '/')
				{
					s.setLength(0);
					while((i = br.read()) != '>')
					{
						s.append((char)i);
					}
					if(s.toString().equalsIgnoreCase("badgeVector"))
					{
						s.setLength(0);
						return;
					}
					else if(s.toString().equals("badge"))
					{
						continue;
					}
					else
					{
						throw new InvalidFileFormatException("Invalid File Format");
					}
				}
				s.setLength(0);
				s.append((char)i);
				while((i = br.read()) != '>')
				{
					s.append((char)i);
				}
				bracket--;
				if(s.toString().equalsIgnoreCase("badge"))
				{
					s.setLength(0);
					while((i = br.read()) != '<')
					{
						s.append((char)i);
					}
					p.addBadge(s.toString());
				}
				else
				{
					throw new InvalidFileFormatException("Invalid File Format");
				}
			}
		}
	}

}