import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.io.*;
/**
 * Defines the output of a pml from a Player
 * 
 * @see Player
 * @see People
 * @see PlayerLoader
 */
public class PlayerSaver
{
	/**
	 * The Player
	 */
	Player p;
	/**
	 * True iff Finished, false otherwise
	 */
	protected boolean fin;
	/**
	 * The String to create
	 */
	StringBuffer s;

	/**
	 * Start creating the pml file based on 
	 * the Player specified.
	 * 
	 * @param p the Player
	 */
	public PlayerSaver(Player p)
	{
		this.p = p;
		s = new StringBuffer();
		startWriting();
	}
	/**
	 * Gets the pml created from the Player.
	 * 
	 * @return the pml file
	 */
	public String toString()
	{
		return s.toString();
	}
	/**
	 * Starting point
	 */
	private void startWriting()
	{
		s.append("<?Player>\n");
		s.append("\t<name>" + p.getName() + "</name>\n");
		s.append("\t<money>" + p.getMoney() + "</money>\n");
		s.append(handleMonsterVector());
		s.append(handleItemVector());
		s.append(handleBadgeVector());
		s.append("</player>");
	}
	/**
	 * Handle the MonsterVector
	 * 
	 * @return the pml for the vector
	 */
	private String handleMonsterVector()
	{
		Monster m;
		StringBuffer sb = new StringBuffer("\t<monsterVector type=home>\n");
		for(int i = 0; i < p.hmon.size(); i++)
		{
			m = (Monster)p.hmon.get(i);
			sb.append(handleMonster(m));
		}
		sb.append("\t</monsterVector>\n");
		sb.append("\t<monsterVector type=hand>\n");
		for(int i = 0; i < p.mon.size(); i++)
		{
			m = (Monster)p.mon.get(i);
			sb.append(handleMonster(m));
		}
		sb.append("\t</monsterVector>\n");
		return sb.toString();
	}
	/**
	 * Handle the Monster
	 * 
	 * @param m the Monster to handle
	 * @return the pml for the Monster
	 */
	private String handleMonster(Monster m)
	{
		StringBuffer sb = new StringBuffer();
		sb.append("\t\t<monster>\n");
		String attacks[] = m.getAttacks();
		switch(m.getKind())
		{
			case Monster.ELECTRIC:
			{
				sb.append("\t\t\t<kind>Electric</kind>\n");
				sb.append("\t\t\t<name>" + m.getName() + "</name>\n");
				sb.append("\t\t\t<hp>" + m.getHP() + "</hp>\n");
				sb.append("\t\t\t<defp>" + m.getDefense() + "</defp>\n");
				sb.append("\t\t\t<attp>" + m.getAttack() + "</attp>\n");
				sb.append("\t\t\t<exp>" + m.getExperience() + "</exp>\n");
				sb.append("\t\t\t<level>" + m.getLevel() + "</level>\n");
				for(int i = 0; i < attacks.length && attacks[i] != null; i++)
				{
					if(attacks[i].equalsIgnoreCase(ElectricMon.A1))
					{
						sb.append("\t\t\t<attack>A1</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(ElectricMon.A2))
					{
						sb.append("\t\t\t<attack>A2</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(ElectricMon.A3))
					{
						sb.append("\t\t\t<attack>A3</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(ElectricMon.A4))
					{
						sb.append("\t\t\t<attack>A4</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(ElectricMon.A5))
					{
						sb.append("\t\t\t<attack>A5</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(ElectricMon.A6))
					{
						sb.append("\t\t\t<attack>A6</attack>\n");
					}
				}
				break;
			}
			case Monster.EARTH:
			{
				sb.append("\t\t\t<kind>Earth</kind>\n");
				sb.append("\t\t\t<name>" + m.getName() + "</name>\n");
				sb.append("\t\t\t<hp>" + m.getHP() + "</hp>\n");
				sb.append("\t\t\t<defp>" + m.getDefense() + "</defp>\n");
				sb.append("\t\t\t<attp>" + m.getAttack() + "</attp>\n");
				sb.append("\t\t\t<exp>" + m.getExperience() + "</exp>\n");
				sb.append("\t\t\t<level>" + m.getLevel() + "</level>\n");
				for(int i = 0; i < attacks.length && attacks[i] != null; i++)
				{
					if(attacks[i].equalsIgnoreCase(EarthMon.A1))
					{
						sb.append("\t\t\t<attack>A1</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(EarthMon.A2))
					{
						sb.append("\t\t\t<attack>A2</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(EarthMon.A3))
					{
						sb.append("\t\t\t<attack>A3</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(EarthMon.A4))
					{
						sb.append("\t\t\t<attack>A4</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(EarthMon.A5))
					{
						sb.append("\t\t\t<attack>A5</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(EarthMon.A6))
					{
						sb.append("\t\t\t<attack>A6</attack>\n");
					}
				}
				break;
			}
			case Monster.FIRE:
			{
				sb.append("\t\t\t<kind>Fire</kind>\n");
				sb.append("\t\t\t<name>" + m.getName() + "</name>\n");
				sb.append("\t\t\t<hp>" + m.getHP() + "</hp>\n");
				sb.append("\t\t\t<defp>" + m.getDefense() + "</defp>\n");
				sb.append("\t\t\t<attp>" + m.getAttack() + "</attp>\n");
				sb.append("\t\t\t<exp>" + m.getExperience() + "</exp>\n");
				sb.append("\t\t\t<level>" + m.getLevel() + "</level>\n");
				for(int i = 0; i < attacks.length && attacks[i] != null; i++)
				{
					if(attacks[i].equalsIgnoreCase(FireMon.A1))
					{
						sb.append("\t\t\t<attack>A1</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(FireMon.A2))
					{
						sb.append("\t\t\t<attack>A2</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(FireMon.A3))
					{
						sb.append("\t\t\t<attack>A3</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(FireMon.A4))
					{
						sb.append("\t\t\t<attack>A4</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(FireMon.A5))
					{
						sb.append("\t\t\t<attack>A5</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(FireMon.A6))
					{
						sb.append("\t\t\t<attack>A6</attack>\n");
					}
				}
				break;
			}
			case Monster.WATER:
			{
				sb.append("\t\t\t<kind>Water</kind>\n");
				sb.append("\t\t\t<name>" + m.getName() + "</name>\n");
				sb.append("\t\t\t<hp>" + m.getHP() + "</hp>\n");
				sb.append("\t\t\t<defp>" + m.getDefense() + "</defp>\n");
				sb.append("\t\t\t<attp>" + m.getAttack() + "</attp>\n");
				sb.append("\t\t\t<exp>" + m.getExperience() + "</exp>\n");
				sb.append("\t\t\t<level>" + m.getLevel() + "</level>\n");
				for(int i = 0; i < attacks.length && attacks[i] != null; i++)
				{
					if(attacks[i].equalsIgnoreCase(WaterMon.A1))
					{
						sb.append("\t\t\t<attack>A1</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(WaterMon.A2))
					{
						sb.append("\t\t\t<attack>A2</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(WaterMon.A3))
					{
						sb.append("\t\t\t<attack>A3</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(WaterMon.A4))
					{
						sb.append("\t\t\t<attack>A4</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(WaterMon.A5))
					{
						sb.append("\t\t\t<attack>A5</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(WaterMon.A6))
					{
						sb.append("\t\t\t<attack>A6</attack>\n");
					}
				}
				break;
			}
			case Monster.GAS:
			{
				sb.append("\t\t\t<kind>Gas</kind>\n");
				sb.append("\t\t\t<name>" + m.getName() + "</name>\n");
				sb.append("\t\t\t<hp>" + m.getHP() + "</hp>\n");
				sb.append("\t\t\t<defp>" + m.getDefense() + "</defp>\n");
				sb.append("\t\t\t<attp>" + m.getAttack() + "</attp>\n");
				sb.append("\t\t\t<exp>" + m.getExperience() + "</exp>\n");
				sb.append("\t\t\t<level>" + m.getLevel() + "</level>\n");
				for(int i = 0; i < attacks.length && attacks[i] != null; i++)
				{
					if(attacks[i].equalsIgnoreCase(GasMon.A1))
					{
						sb.append("\t\t\t<attack>A1</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(GasMon.A2))
					{
						sb.append("\t\t\t<attack>A2</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(GasMon.A3))
					{
						sb.append("\t\t\t<attack>A3</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(GasMon.A4))
					{
						sb.append("\t\t\t<attack>A4</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(GasMon.A5))
					{
						sb.append("\t\t\t<attack>A5</attack>\n");
					}
					else if(attacks[i].equalsIgnoreCase(GasMon.A6))
					{
						sb.append("\t\t\t<attack>A6</attack>\n");
					}
				}
				break;
			}
		}
		sb.append("\t\t</monster>\n");
		return sb.toString();
	}

	/**
	 * Handle the Item Vector
	 * 
	 * @param the pml for the Item Vector
	 */
	private String handleItemVector()
	{
		Item it;
		StringBuffer sb = new StringBuffer("\t<itemVector>\n");
		for(int i = 0; i < p.items.size(); i++)
		{
			it = (Item)p.items.get(i);
			sb.append(handleItem(it));
		}
		sb.append("</itemVector>\n");
		return sb.toString();
	}
	/**
	 * Handle the Item specified
	 * 
	 * @param it the Item to handle
	 * @return the pml for the Item
	 */
	private String handleItem(Item it)
	{
		StringBuffer sb = new StringBuffer("\t\t<item>\n");
		String kind = null;
		switch(it.getBrand())
		{
			case Item.POTION:
			{
				switch(((PotionItem)it).getSubBrand())
				{
					case PotionItem.HPUPFULL:
					{
						kind = "HPUPFULL";
						break;
					}
					case PotionItem.HPUPHALF:
					{
						kind = "HPUPHALF";
						break;
					}
					case PotionItem.HPUPTHIRD:
					{
						kind = "HPUPTHIRD";
						break;
					}
					case PotionItem.ATTUP:
					{
						kind = "ATTUP";
						break;
					}
					case PotionItem.DEFUP:
					{
						kind = "DEFUP";
						break;
					}
				}
				sb.append("\t\t\t<potion>" + kind + "</potion>\n");
				break;
			}
			case Item.MONSTERBALL:
			{
				switch(((MonsterBallItem)it).getSubBrand())
				{
					case MonsterBallItem.NORMAL:
					{
						kind = "Normal";
						break;
					}
					case MonsterBallItem.MASTER:
					{
						kind = "MASTER";
						break;
					}
				}
				sb.append("\t\t\t<monsterBall>" + kind + "</monsterBall>\n");
				break;
			}
		}
		sb.append("\t\t\t<quantity>" + it.getItemCount() + "</quantity>\n");
		sb.append("\t\t</item>\n");
		return sb.toString();
	}
	/**
	 * Handle the Badge Vector
	 * 
	 * @return the pml for the Badge Vector
	 */
	private String handleBadgeVector()
	{
		StringBuffer sb = new StringBuffer("\t<badgeVector>\n");
		for(int i = 0; i < p.badges.size(); i++)
		{
			sb.append("\t\t<badge>" + (String)p.badges.get(i) + "</badge>\n");
		}
		sb.append("\t</badgeVector>\n");
		return sb.toString();
	}

	/**
	 * Testing the Methods.
	 */
	public static void main(String args[])
	{
		try
		{
			BufferedReader in = new BufferedReader(new FileReader("maps/Default.pml"));
			PlayerLoader pload = new PlayerLoader(in);
			Player pl = pload.getPlayer();
			PlayerSaver ps = new PlayerSaver(pl);
			StringBuffer s = new StringBuffer(ps.toString());
			System.out.println(s.toString());
		}
		catch(Exception ee) { ee.printStackTrace(); }
	}
}