import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;
/**
 * Class to define the Potions, HP Up's, Attack Up's, and Defense Up's.
 * 
 * @see Item
 * @see People 
 */
public class PotionItem extends Item
{
	/**
	 * The HP Up half subBrand Identifier
	 */
	public static final int HPUPHALF = 4;
	/**
	 * The Defense Up subBrand Identifier
	 */
	public static final int DEFUP = 5;
	/**
	 * The Attack Up subBrand Identifier
	 */
	public static final int ATTUP = 6;
	/**
	 * The HP Up full subBrand Identifier
	 */
	public static final int HPUPFULL = 7;
	/**
	 * The HP Up third subBrand Identifier
	 */
	public static final int HPUPTHIRD = 8;

	/**
	 * The type
	 */
	private int subBrand;

	/**
	 * Create a new PotionItem with the specified name, and subBrand.
	 * 
	 * @param name the name of the Item
	 * @param subBrand the type of Item defined above
	 */
	public PotionItem(String name, int subBrand)
	{
		if(name != null)
			this.name = name;
		else
			this.name = "Potion";
		brand = POTION;
		if(subBrand == HPUPHALF || subBrand == DEFUP || subBrand == ATTUP || subBrand == HPUPFULL || subBrand == HPUPTHIRD)
			this.subBrand = subBrand;
		else
			this.subBrand = HPUPTHIRD;
		number = 1;
	}
	/**
	 * Returns the type of Item this is.
	 * 
	 * @return the type of Item this is.
	 */
	public int getSubBrand() { return subBrand; }

	/**
	 * Use this on the specified Monster
	 * 
	 * @param m The Monster to use this on, should be the Players Monster.
	 * @return true
	 */
	public boolean useItem(Monster m) // Your Monster
	{
		int i = m.getKind();
		number--;
		switch(subBrand)
		{
			case HPUPHALF:
			{
				m.setHP(m.getHP() + m.getMaxHP()/2);
				break;
			}
			case HPUPFULL:
			{
				m.setHP(m.getMaxHP());
				break;
			}
			case HPUPTHIRD:
			{
				m.setHP(m.getHP() + m.getMaxHP()/3);
				break;
			}
			case DEFUP:
			{
				if(i == m.ELECTRIC)
				{
					m.setMaxDefense(m.getMaxDefense() + 7);
					m.setDefense(m.getMaxDefense());
				}
				else if(i == m.EARTH)
				{
					m.setMaxDefense(m.getMaxDefense() + 6);
					m.setDefense(m.getMaxDefense());
				}
				else if(i == m.WATER)
				{
					m.setMaxDefense(m.getMaxDefense() + 10);
					m.setDefense(m.getMaxDefense());
				}
				else if(i == m.FIRE)
				{
					m.setMaxDefense(m.getMaxDefense() + 7);
					m.setDefense(m.getMaxDefense());
				}
				else if(i == m.GAS)
				{
					m.setMaxDefense(m.getMaxDefense() + 10);
					m.setDefense(m.getMaxDefense());
				}
				break;
			}
			case ATTUP:
			{
				if(i == m.ELECTRIC)
				{
					m.setMaxAttack(m.getMaxAttack() + 10);
					m.setAttack(m.getMaxAttack());
				}
				else if(i == m.EARTH)
				{
					m.setMaxAttack(m.getMaxAttack() + 7);
					m.setAttack(m.getMaxAttack());
				}
				else if(i == m.FIRE)
				{
					m.setMaxAttack(m.getMaxAttack() + 6);
					m.setAttack(m.getMaxAttack());
				}
				else if(i == m.WATER)
				{
					m.setMaxAttack(m.getMaxAttack() + 7);
					m.setAttack(m.getMaxAttack());
				}
				else if(i == m.GAS)
				{
					m.setMaxAttack(m.getMaxAttack() + 6);
					m.setAttack(m.getMaxAttack());
				}
				break;
			}
		}
		return true;
	}
}