import java.awt.*;
import javax.swing.*;
import java.applet.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
/**
 * Class to define the Redirectors to direct to
 * other maps.
 * 
 * @see MapLoader
 * @see Game
 * @see MapObject
 * @see Grid
 */
public class Redirect extends MapObject
{
	/**
	 * The Image to display
	 */
	protected static Image img;
	/**
	 * The Number of Badges required before can be used
	 */
	protected int numBadgesReq;
	/**
	 * filename of where to go
	 */
	String mapTo;
	/**
	 * The Default MapObject to display behind.
	 * 
	 * @see MapObject
	 * @see Grid
	 */
	static MapObject behind;
	/**
	 * Default constructor, sets Map to Load
	 * to Saphire.
	 */
	public Redirect()
	{
		enterable = false;
		type = MapObject.REDIRECT;
		mapTo = "Saphire";
		numBadgesReq = 0;
	}
	/**
	 * Create a new Redirector pointing to the map
	 * specified.
	 * 
	 * @param mapTo the map to load.
	 */
	public Redirect(String mapTo)
	{
		enterable = false;
		type = MapObject.REDIRECT;
		if(mapTo != null)
			this.mapTo = mapTo;
		else
			this.mapTo = "Saphire";
		numBadgesReq = 0;
	}
	/**
	 * Sets the image to paint.
	 * 
	 * @param img the array of images to load, needs to be of length 1
	 */
	public void setImage(Image[] img)
	{
		if(img.length == 1)
			this.img = img[0];
	}
	/**
	 * Sets the map to load.
	 * 
	 * @param mapTo the Map to load.
	 */
	public void setMapTo(String mapTo)
	{
		if(mapTo != null)
			this.mapTo = mapTo;
	}
	/**
	 * Gets the map to load.
	 * 
	 * @return the map to load
	 */
	public String getMapTo() { return mapTo; }

	/**
	 * Sets the number of Badges required to use this.
	 * 
	 * @param numBadges the NUmber of Badges required to use this
	 */
	public void setNumberBadgesRequired(int numBadges)
	{
		if(numBadges >= 0)
			numBadgesReq = numBadges;
	}
	/**
	 * Gets the number of badges required to use this.
	 * 
	 * @return the number of badges required to use this
	 */
	public int getNumberBadgesRequired() { return numBadgesReq; }
	/**
	 * returns a clone of this
	 * 
	 * @return a clone of this
	 */
	public MapObject getClone() { return new Redirect(mapTo); }
	/**
	 * Sets the MapObject that this is over.
	 * 
	 * @param m the MapObject this is over.
	 */
	public void setBehind(MapObject m)
	{
		if(m == null)
			return;
		else if(m.canBeEntered())
		{
			this.behind = m;
		}
	}
	/**
	 * Draws the redirector.
	 * 
	 * @param g the Graphics.
	 */
	public void paint(Graphics g)
	{
		behind.paint(g);
		g.drawImage(img, 0, 0, this);
	}
}