import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.util.*;
/**
 * Simple Event to tell Game to load a new Map.
 * 
 * @see Game
 * @see Grid
 * @see MapLoader
 * @see Redirect
 */
public class RedirectEvent extends AWTEvent 
{
	/**
	 * The filename
	 */
	String mapName;
	/**
	 * The Player
	 */
	Player p;
	/**
	 * the UID
	 */
	public static final int REDIRECT_FIRED = RESERVED_ID_MAX + 10;	// Make sure no conflicts with normal AWT components

	/**
	 * Construct a new Event with the Grid and Player.
	 * 
	 * @param source the Grid
	 * @param player the Player
	 * @see Grid
	 * @see Redirect
	 * @see Player
	 */
	RedirectEvent(Grid source, int id, String mapName, Player player)
	{
		super( source, id );
		this.mapName = mapName;
		p = player;
	}
	/**
	 * Returns the filename for the new map.
	 * 
	 * @return the filename for the new map.
	 */
	public String getMapName() { return mapName; }
	/**
	 * Returns the Player to be placed on the new map.
	 * 
	 * @return the Player to be placed on the new map..
	 */
	public Player getPlayer() { return p; }
}