import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.awt.image.*;

/**
 * Defines the Shop where the Player can buy Items, Save
 * Heal his Monsters, and Switch Monsters from Home to his Hand.
 * 
 * @see ShopEndEvent
 * @see SaveEvent
 * @see Player
 * @author Matt, Sam
 * @version 4.5.7.8
 */

public class Shop extends Component 
{
	/**
	 * The Player to work with
	 */
	Player p;
	/**
	 * The Image of the Shop
	 */
	Image bg;

	/**
	 * The Listeners for ShopEndEvent
	 */
	Vector shopEndListeners;
	/**
	 * The listeners for SaveEvent
	 */
	Vector saveGameListeners;

	// The positions for drawing
	/**
	 * The starting x-position
	 */
	private static final int XST = 47;
	/**
	 * The starting y-position for shelf 1
	 */
	private static final int YST = 43;
	/**
	 * The starting y-position for shelf 2
	 */
	private static final int YST2 = 121;
	/**
	 * The width of the Items plus change
	 */
	private static final int XWT = 54;
	/**
	 * The x temporal transition
	 */
	private static final int XTT = 27;
	/**
	 * The y height for items
	 */
	private static final int YWT = 50;
	// The Prices for items
	/**
	 * The price for HP Up Full
	 */
	private static final int HPUPF = 500;
	/**
	 * The price for HP Up Half
	 */
	private static final int HPUPH = 250;
	/**
	 * The price for HP Up Third
	 */
	private static final int HPUPT = 167;
	/**
	 * The price for Defense Up
	 */
	private static final int DEFUP = 200;
	/**
	 * The price for Attack Up
	 */
	private static final int ATTUP = 200;
	/**
	 * The price for Normal MonsterBalls
	 */
	private static final int NORMA = 150;
	/**
	 * The price for Master MonsterBalls
	 */
	private static final int MASTE = 300;

	// What is being done.
	/**
	 * Constant for buying
	 */
	private static final int BUY = 0;
	/**
	 * Constant for Shopping
	 */
	private static final int SHOP = 1;
	/**
	 * onstant for Switching
	 */
	private static final int SWITCH = 2;
	/**
	 * Constant for Healing
	 */
	private static final int HEAL = 3;
	/**
	 * Constant for Saving
	 */
	private static final int SAVE = 4;
	/**
	 * Constant for Leaving
	 */
	private static final int LEAVE = 5;
	/**
	 * Constant for Switching Hand Portion
	 */
	private static final int SWITCHH = 6;

	/**
	 * Many int needed to speciy drawing regions
	 */
	int shelf, sel, sel2, xs, ys, ys2, op, start, start2;
	/**
	 * What is displayed to Player
	 */
	String[] disp;
	/**
	 * The Options for the Player
	 */
	String[] ops;

	/**
	 * Construct a Shop Object with a Player and an Image.
	 * 
	 * @param p the Player to use
	 * @param bg the Image of the Shop
	 */
	public Shop(Player p, Image bg)
	{
		this.p = p;
		this.bg = bg;
		sel = 0;
		shelf = 0;
		op = 0;
		start = 0;
		xs = XST;
		ys = YST;

		disp = new String[2];
		disp[0] = "Howdy partner!";
		disp[1] = "How may I help you?";

		ops = new String[5];
		ops[0] = "Buy";
		ops[1] = "Heal";
		ops[2] = "Switch";
		ops[3] = "Save";
		ops[4] = "Leave";

		enableEvents(AWTEvent.KEY_EVENT_MASK);
	}

	/**
	 * Handle Events that are passed into this.
	 * Overrides implementation for KeyEvent.KEY_PRESSED
	 * and KeyEvent.KEY_RELEASED, allows super to deal with the rest
	 * 
	 * @param e the AWTEvent to handle
	 * @see AWTEvent
	 */
	public void processEvent(AWTEvent e)
	{
		if(e.getID() == KeyEvent.KEY_PRESSED)
		{
			switch(((KeyEvent)e).getKeyCode())
			{
				case KeyEvent.VK_LEFT:
				{
					if(op == BUY)
					{
						if(sel != 0)
						{
							sel--;
							xs += XWT + XTT;
							repaint(7, 343, 600, 100);
						}
						else
						{
							sel = ops.length-1;
							xs = XST;
							repaint(7, 343, 600, 100);
						}
					}
					else if(op == SWITCH || op == SWITCHH)
					{
						if(sel == 0)
						{
							if(start == 0)
							{
								if(ops.length - 5 >= 0)
									start = ops.length-5;
								else
									start = 0;
								sel = ops.length-1-start;
							}
							else if( (start - 5) >= 0)
							{
								start -= 5;
								sel = 0;
							}
							else
							{
								start = 0;
								sel = 0;
							}
							xs = XST;
						}
						else
						{
							sel--;
							xs += XWT + XTT;
						}
						repaint(7, 343, 600, 100);
					}
					else if(op == SHOP)
					{
						if(shelf == 0)
						{
							if(sel == 0)
							{
								sel = 4;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = (XST + (XWT + XTT) * 4);
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
							else
							{
								sel--;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs -= XWT + XTT;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
						}
						else
						{
							if(sel == 0)
							{
								sel = 1;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = XST + XWT + XTT;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
							else
							{
								sel--;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = XST;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
						}
					}
					break;
				}
				case KeyEvent.VK_RIGHT:
				{
					if(op == BUY)
					{
						if(sel == ops.length-1)
						{
							start = 0;
							sel = 0;
						}
						else
						{
							sel++;
						}
						repaint(7, 343, 600, 100);
					}
					else if(op == SWITCH || op == SWITCHH)
					{
						if((sel+start) == ops.length-1)
						{
							start = 0;
							sel = 0;
						}
						else if(sel == 4)
						{
							if((start+5) < ops.length)
							{
								start += 5;
								sel = 0;
							}
							else
							{
								start = ops.length-5;
								sel = 0;
							}
						}
						else
							sel++;
						repaint(7, 343, 600, 100);
					}
					else if(op == SHOP)
					{
						if(shelf == 0)
						{
							if(sel == 4)
							{
								sel = 0;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = XST;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
							else
							{
								sel++;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs += XWT + XTT;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
						}
						else
						{
							if(sel == 1)
							{
								sel = 0;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = XST;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
							else
							{
								sel++;
								repaint(xs, ys, XWT+XTT, YWT+7);
								xs = XST + XWT + XTT;
								repaint(xs, ys, XWT+XTT, YWT+7);
							}
						}
					}
					break;
				}
				case KeyEvent.VK_DOWN:
				{
					if(op == SHOP)
					{
						sel = 0;
						if(shelf == 0)
						{
							shelf++;
							repaint(xs, ys, XWT+XTT, YWT+7);
							xs = XST;
							ys = YST2;
							repaint(xs, ys, XWT+XTT, YWT+7);
						}
						else
						{
							shelf = 0;
							repaint(xs, ys, XWT+XTT, YWT+7);
							xs = XST;
							ys = YST;
							repaint(xs, ys, XWT+XTT, YWT+7);
						}
					}
					break;
				}
				case KeyEvent.VK_UP:
				{
					if(op == SHOP)
					{
						sel = 0;
						if(shelf == 1)
						{
							shelf--;
							repaint(xs, ys, XWT+XTT, YWT+7);
							xs = XST;
							ys = YST;
							repaint(xs, ys, XWT+XTT, YWT+7);
						}
						else
						{
							shelf = 1;
							repaint(xs, ys, XWT+XTT, YWT+7);
							xs = XST;
							ys = YST2;
							repaint(xs, ys, XWT+XTT, YWT+7);
						}
					}
					break;
				}
				case KeyEvent.VK_ENTER:
				case KeyEvent.VK_SPACE:
				{
					repaint(xs, ys, XWT+XTT, YWT+7);
					if(op == BUY)
					{
						if(sel == 4)
							fireEvent(p);
						else if(sel == 0)
						{
							shelf = 0;
							sel = 0;
							start = 0;
							disp = new String[2];
							disp[0] = "Please Choose a Selection.";
							disp[1] = "You have: $" + p.getMoney();
							ops = new String[1];
							ops[0] = "";
							op = SHOP;
						}
						else if(sel == 1)
						{
							p.healAllMonsters();
							disp = new String[2];
							disp[0] = "All Your Monsters have been healed";
							disp[1] = "Good Luck Pardner.";
							ops = new String[1];
							ops[0] = "";
							op = HEAL;
						}
						else if(sel == 2)
						{
							int num = p.getNumberOfMonstersAtHome();
							if(num > 0)
							{
								sel = 0;
								shelf = 0;
								start = 0;
								disp = new String[2];
								disp[0] = "Please choose the Monster from";
								disp[1] = "home to switch into your hand";
								ops = new String[num];
								System.out.println(num);
								for(int i = 0; i < num; i++)
								{
									ops[i] = p.getMonsterAtHome(i).getName();
								}
								op = SWITCH;
							}
							else
							{
								sel = 0;
								shelf = 0;
								start = 0;
								xs = XST;
								ys = YST;

								disp = new String[2];
								disp[0] = "Howdy partner!";
								disp[1] = "How may I help you?";

								ops = new String[5];
								ops[0] = "Buy";
								ops[1] = "Heal";
								ops[2] = "Switch";
								ops[3] = "Save";
								ops[4] = "Leave";
								op = BUY;
							}
						}
						else if(sel == 3)
						{
							fireSaveEvent(p);

							sel = 0;
							shelf = 0;
							start = 0;
							xs = XST;
							ys = YST;

							disp = new String[2];
							disp[0] = "Howdy partner!";
							disp[1] = "How may I help you?";

							ops = new String[5];
							ops[0] = "Buy";
							ops[1] = "Heal";
							ops[2] = "Switch";
							ops[3] = "Save";
							ops[4] = "Leave";
							op = BUY;
						}
					}
					else if(op == HEAL)
					{
						sel = 0;
						shelf = 0;
						start = 0;
						xs = XST;
						ys = YST;

						disp = new String[2];
						disp[0] = "Howdy partner!";
						disp[1] = "How may I help you?";

						ops = new String[5];
						ops[0] = "Buy";
						ops[1] = "Heal";
						ops[2] = "Switch";
						ops[3] = "Save";
						ops[4] = "Leave";
						op = BUY;
					}
					else if(op == SWITCH)
					{
						sel2 = start + sel;
						sel = 0;
						start = 0;
						disp = new String[2];
						disp[0] = "Please choose the Monster from";
						disp[1] = "your hand to send home";
						int num = p.getNumberOfMonsters();
						if(num > 0)
						{
							ops = new String[num];
							for(int i = 0; i < num; i++)
							{
								ops[i] = p.getMonster(i).getName();
							}
							op = SWITCHH;
						}
						else
						{
							shelf = 0;
							xs = XST;
							ys = YST;

							disp = new String[2];
							disp[0] = "Howdy partner!";
							disp[1] = "How may I help you?";

							ops = new String[5];
							ops[0] = "Buy";
							ops[1] = "Heal";
							ops[2] = "Switch";
							ops[3] = "Save";
							ops[4] = "Leave";
							op = BUY;
						}
					}
					else if(op == SWITCHH)
					{
						p.switchMonster(p.getMonster(sel), p.getMonsterAtHome(sel2));
						sel = 0;
						sel2 = 0;
						shelf = 0;
						start = 0;
						xs = XST;
						ys = YST;

						disp = new String[2];
						disp[0] = "Howdy partner!";
						disp[1] = "How may I help you?";

						ops = new String[5];
						ops[0] = "Buy";
						ops[1] = "Heal";
						ops[2] = "Switch";
						ops[3] = "Save";
						ops[4] = "Leave";
						op = BUY;
					}
					else if(op == SHOP)
					{
						if(checkMoney())
						{
							sel = 0;
							shelf = 0;
							start = 0;
							xs = XST;
							ys = YST;

							disp = new String[2];
							disp[0] = "Howdy partner!";
							disp[1] = "How may I help you?";

							ops = new String[5];
							ops[0] = "Buy";
							ops[1] = "Heal";
							ops[2] = "Switch";
							ops[3] = "Save";
							ops[4] = "Leave";
							op = BUY;
						}
						else
						{
							disp = new String[3];
							disp[0] = "Now See Here!";
							disp[1] = "Ya'all ain't got enough money, partner.";
							disp[2] = "Press escape to continue.";
						}
					}
					repaint(7, 343, 600, 100);
					break;
				}
				case KeyEvent.VK_ESCAPE:
				{
					repaint(xs, ys, XWT+XTT, YWT+7);
					sel = 0;
					sel2 = 0;
					shelf = 0;
					start = 0;
					xs = XST;
					ys = YST;
					op = BUY;

					disp = new String[2];
					disp[0] = "Howdy partner!";
					disp[1] = "How may I help you?";

					ops = new String[5];
					ops[0] = "Buy";
					ops[1] = "Heal";
					ops[2] = "Switch";
					ops[3] = "Save";
					ops[4] = "Leave";
					repaint(7, 343, 600, 100);
					break;
				}
			}
		}
		super.processEvent(e);
	}

	/**
	 * Overriden to only call paint without first
	 * painting a background first, removes the flicker effect
	 * 
	 * @param g the Graphics
	 */
	public void update(Graphics g)
	{
		paint(g);
	}
	/**
	 * Draw the Shop, and handle selection highlighting
	 * 
	 * @param g the Graphics
	 */
	public void paint(Graphics g)
	{
		g.setColor(Color.white);
		g.fillRect(0, 0, 640, 450);
		g.drawImage(bg, 0, 0, this);

		drawText(g);

		if(op == SHOP)
		{
			g.setColor(Color.red);
			g.drawRect(xs, ys, XWT, YWT);
		}
	}

	/**
	 * Draws the text that is presented to the Player
	 * 
	 * @param g the Graphics
	 */
	private void drawText(Graphics g)
	{
		int xp = 7;
		int yp = 343;

		g.setColor(Color.white);
		g.fillRect(xp, yp, 600, 100);

		g.setColor(Color.black);
		int ym = yp+20;
		for(int i = 0; i < disp.length; i++)
		{
			g.drawString(disp[i], xp+8, ym+8);
			ym += g.getFontMetrics().getHeight();
		}
		for(int i = start, j = 0, xm = xp+10; i < ops.length && j < 5; i++, j++)
		{
			if(i == sel+start)
				g.setColor(Color.red);
			else
				g.setColor(Color.black);
			g.drawString(ops[i], xm+8, ym+8);
			xm += g.getFontMetrics().stringWidth(ops[i]) + 10;
		}
	}

	/**
	 * Add a Class to fire ShopEndEvents into
	 * 
	 * @param bl the Class that wishes to be informed
	 * @see ShopEndListener
	 * @see ShopEndEvent
	 */
	public void addShopEndListener(ShopEndListener bl)
	{
		if ( shopEndListeners == null )
			shopEndListeners = new Vector();
		shopEndListeners.addElement( bl );
	}
	/**
	 * Removes a Class to fire ShopEndEvents into
	 * 
	 * @param bl the Class that wishes to not be informed anymore
	 * @see ShopEndListener
	 * @see ShopEndEvent
	 */
	public void removeShopListener(ShopEndListener bl)
	{
		if ( shopEndListeners != null )
			shopEndListeners.removeElement( bl );
	}
	/**
	 * Add a Class to fire SaveEvent into
	 * 
	 * @param bl the Class that wishes to be informed
	 * @see SaveListener
	 * @see SaveEvent
	 */
	public void addSaveListener(SaveListener bl)
	{
		if ( saveGameListeners == null )
			saveGameListeners = new Vector();
		saveGameListeners.addElement( bl );
	}
	/**
	 * Removes a Class to fire SaveEvent into
	 * 
	 * @param bl the Class that wishes to not be informed anymore
	 * @see SaveListener
	 * @see SaveEvent
	 */
	public void removeSaveListener(SaveEvent bl)
	{
		if ( saveGameListeners != null )
			saveGameListeners.removeElement( bl );
	}
	/**
	 * Fires the ShopEndEvent to all listeners
	 * 
	 * @param p the Player to pass to listeners
	 */
	private void fireEvent(Player p) 
	{
		if ( shopEndListeners == null )
			return;
		ShopEndEvent event = new ShopEndEvent(this, ShopEndEvent.SHOP_ENDED, p);
		for (Enumeration e = shopEndListeners.elements(); e.hasMoreElements(); )
			((ShopEndListener)e.nextElement()).shopEnd( event );
	}

	/**
	 * Fires the SaveEvent to all listeners
	 * 
	 * @param p the Player to pass to listeners
	 */
	private void fireSaveEvent(Player p)
	{
		if ( saveGameListeners == null )
			return;
		SaveEvent event = new SaveEvent(this, SaveEvent.SAVE_GAME, p);
		for (Enumeration e = saveGameListeners.elements(); e.hasMoreElements(); )
			((SaveListener)e.nextElement()).saveGame( event );
	}
	/**
	 * Determines if the Player has enough money
	 * to buy the Item selected.
	 * 
	 * @return true iff Player has enough money for Item selected
	 */
	private boolean checkMoney()
	{
		int mon = p.getMoney();
		switch(shelf)
		{
			case 0: // Add a potion
			{
				switch(sel)
				{
					case 0:
					{
						if(mon >= HPUPF)
						{
							p.setMoney(mon - HPUPF);
							p.addItem(new PotionItem("HP Up Full", PotionItem.HPUPFULL));
							return true;
						}
						return false;
					}
					case 1:
					{
						if(mon >= HPUPH)
						{
							p.setMoney(mon - HPUPH);
							p.addItem(new PotionItem("HP Up Half", PotionItem.HPUPHALF));
							return true;
						}
						return false;
					}
					case 2:
					{
						if(mon >= HPUPT)
						{
							p.setMoney(mon- HPUPT);
							p.addItem(new PotionItem("HP Up Third", PotionItem.HPUPTHIRD));
							return true;
						}
						return false;
					}
					case 3:
					{
						if(mon >= ATTUP)
						{
							p.setMoney(mon - ATTUP);
							p.addItem(new PotionItem("Attack Up", PotionItem.ATTUP));
							return true;
						}
						return false;
					}
					case 4:
					{
						if(mon >= DEFUP)
						{
							p.setMoney(mon - DEFUP);
							p.addItem(new PotionItem("Defense Up", PotionItem.DEFUP));
							return true;
						}
						return false;
					}
				}
				break;
			}
			case 1: // Add a Ball
			{
				switch(sel)
				{
					case 0:
					{
						if(mon >= NORMA)
						{
							p.setMoney(mon - NORMA);
							p.addItem(new MonsterBallItem("Normal MonsterBall", MonsterBallItem.NORMAL));
							return true;
						}
						return false;
					}
					case 1:
					{
						if(mon >= MASTE)
						{
							p.setMoney(mon - MASTE);
							p.addItem(new MonsterBallItem("Master MonsterBall", MonsterBallItem.MASTER));
							return true;
						}
						return false;
					}
				}
				break;
			}
		}
		return false;
	}
}