import java.awt.*;
import java.applet.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.net.*;
import java.util.*;

/**
 * Implementation of a Trainer, a Person with a Badge,
 * and not able to move around the Grid.  Name still needs to be 
 * unique for a Grid as this is used for equals.
 * 
 * @see Person
 * @see People
 * @see Grid
 * @author Matt, Sam
 * @version 1.2.5.8
 */
public class Trainer extends People implements Runnable
{
	/**
	 * The Images associated with the Four directions,
	 * Up, Left, Down, Right/
	 */
	protected static Image img, bimg, limg, rimg; // The Four directions
	/**
	 * The name of the badge
	 */
	protected String badge;
	/**
	 * Where I am on the Grid
	 */
	int pi, pj;
	/**
	 * The Grid I am on
	 */
	protected Grid g;
	/**
	 * My Thread Object
	 */
	Thread t;
	
	/**
	 * Constructs a default Trainer
	 * No money,
	 * Grass as behind MapObject
	 * Name of NPC
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 */
	public Trainer()
	{
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "NPC";
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		dlg = "";
	}
	/**
	 * Constructs a Trainer facing in direction dir
	 * No money,
	 * Grass as behind MapObject
	 * Name of NPC
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 * 
	 * @param dir the direction to face, specified in Grid
	 * @see Grid
	 */
	public Trainer(int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		behind = new Grass();
		name = "NPC";
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
		dlg = "";
	}
	/**
	 * Constructs a Trainer with MapObject underneath
	 * No money,
	 * Name of NPC
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 * 
	 * @param behind The MapObject standing on
	 */
	public Trainer(MapObject behind)
	{
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		name = "NPC";
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Constructs a Trainer with MapObject underneath
	 * and facing dir.
	 * No money,
	 * Name of NPC
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 * 
	 * @param behind The MapObject standing on
	 * @param dir the Direction facing, specified in Grid
	 * @see Grid
	 */
	public Trainer(MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		name = "NPC";
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Constructs a Trainer with Name name.
	 * No money,
	 * Grass as behind MapObject
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 * 
	 * @param name The name of the Trainer
	 */
	public Trainer(String name)
	{
		this.dir = Grid.DOWN;
		this.behind = new Grass();
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		this.name = name;
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}
	/**
	 * Constructs a Trainer with Name name,
	 * Standing on behind, and facing dir.
	 * No money,
	 * Badge of Name
	 * No Monsters
	 * No Dialogue
	 * 
	 * @param name The Name of the Trainer
	 * @param behind The MapObject standing on
	 * @param dir The direction facing, specified in Grid
	 * @see Grid
	 */
	public Trainer(String name, MapObject behind, int dir)
	{
		if(!setDirection(dir))
			this.dir = Grid.DOWN;
		if(behind.canBeEntered())
			this.behind = behind;
		else
			this.behind = new Grass();
		enterable = false;
		type = MapObject.TRAINER;
		activeMon = 0;
		money = 0;
		this.name = "name";
		badge = name;
		mon = new Vector();
		hmon = new Vector();
		items = new Vector();
	}

	/**
	 * Sets the static Image for this class.
	 * Requires a call to this before a call to paint.
	 * 
	 * @param img the Image of the Trainer
	 */
	public void setImage(Image[] img)
	{
		if(img.length == 4)
		{
			this.img = img[0];
			this.bimg = img[1];
			this.limg = img[2];
			this.rimg = img[3];
		}
	}
	/**
	 * Sets the name of the Trainer.
	 * 
	 * @param name The Name of the Trainer, must be unique on a Grid.
	 */
	public void setName(String name)
	{
		this.name = name;
	}
	/**
	 * Sets the name of the badge carried by the Trainer.
	 * 
	 * @param name The Name of the Badge carried, set to "" to stop Thread
	 */
	public void setBadge(String bad) { if(bad != null) badge = bad; }
	/**
	 * Gets the name of the Badge carried.
	 * 
	 * @return The name of the badge carried.
	 */
	public String getBadge() { return badge; }
	/**
	 * Returns a clone of this, a Trainer with the same
	 * name and direction facing.
	 * 
	 * @return A new MapObject of this.
	 */
	public synchronized MapObject getClone() { return new Trainer(name, behind, dir); }
	/**
	 * Determines the Player that is next to this, that
	 * are within one square any direction including diagonally.
	 * 
	 * @return The MapObject of the Player near this.
	 * @param i the i'th location in Grid.
	 * @param j the j'th location in Grid.
	 * @see Grid
	 * @see Player
	 * @see People
	 */
	public synchronized MapObject peopleNextTo(int i, int j)
	{
		MapObject mptemp;
		int k = 0;
		for (int x = pi - 1; x <= pi + 1; x++) 
		{
			for (int y = pj - 1; y <= pj + 1; y++) 
			{
				if (x != pi || y != pj) 
				{
					if ( g.isValidLocation(x, y) )
					{
						try
						{
							mptemp = g.mapObjectAt(x, y);
							if(mptemp.getType() == MapObject.PERSON || mptemp.getType() == MapObject.PLAYER)
							{
								if( !( ((People)mptemp).getName().equals(t.getName()) ) && !( ((People)mptemp).isBattling() ) )
									return mptemp;
							}
						}
						catch(Exception e) { } // Normal to have happen
					}
				}
			}
		}
		return null;
	}
	/**
	 * Sets the Grid that this is on, and starts the Thread
	 * for checking if a Player enters the sight of this.
	 * 
	 * @param g the Grid this is on, set to null to stop this Thread
	 */
	public synchronized void setGrid(Grid g)
	{
		this.g = g;
		try
		{
			Location l = g.locationOf(this);
			pi = l.getI();
			pj = l.getJ();
			t = new Thread(this);
			t.start();
		}
		catch(Exception e) { }
	}
	/**
	 * Check if there is a Player to battle, and sleep
	 * for one second between checks.  Ends when this.grid is null or
	 * the name of this.badge is not "".
	 */
	public void run()
	{
		while(g != null && !badge.equals(""))
		{
			if(!battling)
			{
				synchronized(g)
				{
					MapObject mp = peopleNextTo(pi, pj);
					if(mp != null)
					{
						if(mp.getType() == MapObject.PLAYER)
						{
							if(((Player)mp).hasBadge(badge))
							{
								return;
							}
							battling = true;
							g.beginBattleWithPlayer(this);
						}
					}
				}
			}
			try
			{
				Thread.sleep(1000);
			}
			catch(Exception e) { }
		}
	}
	/**
	 * Draws the image associated with Trainer, and the MapObject this is on.
	 * 
	 * @see MapObject
	 */
	public void paint(Graphics g)
	{
		behind.paint(g);

		int wi, he;
		switch(dir)
		{
			case Grid.LEFT:
				{
				g.drawImage(limg, 0, 0, this);
				break;
			}
			case Grid.RIGHT:
				{
				g.drawImage(rimg, 0, 0, this);
				break;
			}
			case Grid.UP:
				{
				g.drawImage(bimg, 0, 0, this);
				break;
			}
			default:
				{
				g.drawImage(img, 0, 0, this);
				break;
			}
		}
	}
}